<?php
namespace App\Http\Controllers;

use App\Address;
use App\DeliveryAddress;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class AddressesController extends Controller
{

    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function getMyAddressList(Request $request)
    {
        $data['addresses'] = Address::where([
            'user_id' => Auth::user()->id
        ])->get();

        return response()->json([
            'status' => 1,
            'message' => 'My Addresses',
            'data' => $data
        ]);
    }

    public function getMyAddressDetail($id)
    {
        $address = Address::where([
            'user_id' => Auth::user()->id,
            'id' => $id
        ])->first();
        if (empty($address)) {
            return response()->json([
                'status' => 0,
                'message' => 'No Address Found',
                'data' => []
            ]);
        } else {
            $data['address'] = $address;
            return response()->json([
                'status' => 1,
                'message' => 'My Address',
                'data' => $data
            ]);
        }
    }

    public function saveMyAddress(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'address1' => 'required',
            'city' => 'required',
            'country' => 'required',
            'zip' => 'required',
            'phone' => 'required'
        ], [
            'name.required' => "Name is required",
            'address1.required' => "Address 1 is required",
            'city.required' => "City is required",
            'country.required' => "Country is required",
            'zip.required' => "Post Code is required",
            'phone.required' => "Phone Number is required"
        ]);

        $fields = array(
            'name',
            'address1',
            'city',
            'country',
            'zip',
            'phone'
        );
        $error_message = "";

        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        $postcode = DeliveryAddress::getPostCodeDetail($request['zip']);
        if (empty($postcode) || $postcode->status != "200") {
            return response()->json([
                'status' => 0,
                'message' => 'Invalid postcode'
            ]);
        }

        $address = new Address();
        $aData = collect($request->all())->put('user_id', Auth::user()->id)->toArray();
        $aData['longitude'] = $postcode->result->longitude;
        $aData['latitude'] = $postcode->result->latitude;

        $address->fill($aData);
        if ($address->save()) {
            return response()->json([
                'status' => 1,
                'message' => 'Address Saved Successfully.',
                'data' => [
                    'address' => Address::find($address->id)
                ]
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'Not Saved'
            ]);
        }
    }

    public function updateMyAddress(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'name' => 'required',
            'address1' => 'required',
            'city' => 'required',
            'country' => 'required',
            'zip' => 'required',
            'phone' => 'required'
        ], [
            'name.required' => "Name is required",
            'address1.required' => "Address 1 is required",
            'city.required' => "City is required",
            'country.required' => "Country is required",
            'zip.required' => "Post Code is required",
            'phone.required' => "Phone Number is required"
        ]);

        $fields = array(
            'id',
            'name',
            'address1',
            'city',
            'country',
            'zip',
            'phone'
        );
        $error_message = "";

        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        $postcode = DeliveryAddress::getPostCodeDetail($request['zip']);
        if (empty($postcode) || $postcode->status != "200") {
            return response()->json([
                'status' => 0,
                'message' => 'Invalid postcode'
            ]);
        }

        $address = Address::where([
            'user_id' => Auth::user()->id,
            'id' => $request['id']
        ])->first();
        if (empty($address)) {
            return response()->json([
                'status' => 0,
                'message' => 'This Address does not belongs to current user.',
                'data' => []
            ]);
        }

        $aData = collect($request->all())->forget('id')
            ->put('user_id', Auth::user()->id)
            ->toArray();
        $aData['latitude'] = $postcode->result->latitude;
        $aData['longitude'] = $postcode->result->longitude;
        $address->fill($aData);

        if ($address->update()) {
            return response()->json([
                'status' => 1,
                'message' => 'Address updated Successfully.',
                'data' => [
                    'address' => Address::find($address->id)
                ]
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'Not Updated'
            ]);
        }
    }

    public function deleteMyAddress(Request $request, $id)
    {
        if (empty($id) || ! is_numeric($id)) {
            return response()->json([
                'status' => 0,
                'message' => 'Address ID is Invalid'
            ]);
        }

        $address = Address::where([
            'user_id' => Auth::user()->id,
            'id' => $id
        ])->first();
        if (empty($address)) {
            return response()->json([
                'status' => 0,
                'message' => 'This Address does not belongs to current user.',
                'data' => []
            ]);
        }

        if ($address->delete()) {
            return response()->json([
                'status' => 1,
                'message' => 'Address deleted Successfully.',
                'data' => [
                    'addresses' => Address::where('user_id', Auth::user()->id)->get()
                ]
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'Not Updated'
            ]);
        }
    }

    public function makeMyAddressDefault(Request $request, $id)
    {
        if (empty($id) || ! is_numeric($id)) {
            return response()->json([
                'status' => 0,
                'message' => 'Address ID is Invalid'
            ]);
        }

        $address = Address::where([
            'user_id' => Auth::user()->id,
            'id' => $id
        ])->first();
        if (empty($address)) {
            return response()->json([
                'status' => 0,
                'message' => 'This Address does not belongs to current user.',
                'data' => []
            ]);
        }

        $address->default = 'Yes';

        DB::beginTransaction();
        $flag = 0;

        try {
            $otherAddresses = Address::where('id', '!=', $id)->where('user_id', Auth::user()->id)->get();
            foreach ($otherAddresses as $otherAddress) {
                $otherAddress->default = "No";
                $otherAddress->save();
            }

            $address->update();

            DB::commit();
            $flag ++;
        } catch (\Exception $e) {
            DB::rollback();
        }

        if ($flag > 0) {
            return response()->json([
                'status' => 1,
                'message' => 'Address Successfully marked as default',
                'data' => [
                    'my_addresses' => Address::where('user_id', Auth::user()->id)->get()
                ]
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'Not Updated'
            ]);
        }
    }

    public function getPostcodeDetail(Request $request)
    {
        if (empty($request['postcode'])) {
            return response()->json([
                'status' => 0,
                'message' => 'Postcode can not be blank.'
            ]);
        }

        $data['postcode_detail'] = DeliveryAddress::getPostCodeDetail($request['postcode']);
        return response()->json([
            'status' => 1,
            'message' => 'Postcode send successfully.',
            'data' => $data
        ]);
    }
}
