<?php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\User;
use Validator;
use Illuminate\Support\Facades\Auth;
use File;
use Intervention\Image\ImageManagerStatic as Image;
use View;
use DB;
use Hash;
use Cache;
use Carbon\Carbon;
use App;
use AWS;
use Illuminate\Support\Facades\Session;
use App\Mail\ResetPasswordMail;
use Illuminate\Support\Facades\Mail;
use App\Mail\SupportMail;

class AuthController extends Controller
{

    public $User;

    public function __construct()
    {
        $this->User = new User();
    }

    
    /* API Actions */
    public static function verifyUser(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required',
            'password' => 'required',
            'device_token' => 'required'
        ]);
        $fields = array(
            'email',
            'password',
            'device_token'
        );

        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        // $user = User::where(['email'=>$request->email,'country_code'=>$request->country_code])->first();

        $user = User::where([
            'email' => $request->email
        ])->first();
        $credentials = request([
            'email',
            'password'
        ]);

        if ($user) {

            // checking if user is blocked.
            if ($user->enabled == 'No') {
                return response()->json([
                    'status' => 2,
                    'message' => 'Sorry! User is Blocked.'
                ]);
            }

            if (Hash::check($request->password, $user->password)) {
                Auth::attempt($credentials);
                $user = Auth::user();

                User::where([
                    'email' => $request->email
                ])->update([
                    'device_token' => $request->device_token
                ]);

                $user->auth_token = $user->createToken('AppName')->accessToken;

                return response()->json([
                    'status' => 1,
                    'message' => 'Login Successful.',
                    'data' => $user
                ]);
            } else {
                return response()->json([
                    'status' => 0,
                    'message' => "Incorect Password"
                ]);
            }
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'User Not Registered.'
            ]);
        }
    }

    function verifyUserSocial(Request $request)
    {
        $data = $request->all();
        
        if($request->is_social == 'no')
        {
            
            $validator = Validator::make($request->all(), [
                'email' => 'required',
                'password' => 'required',
//                 'device_token' => 'required'
            ]);
            $fields = array(
                'email',
                'password',
//                 'device_token'
            );
            
            $error_message = "";
            if ($validator->fails()) {
                foreach ($fields as $field) {
                    if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                        $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                    }
                }
                return response()->json([
                    'status' => 0,
                    'message' => substr($error_message, 0, - 2)
                ]);
            }
            
            // $user = User::where(['email'=>$request->email,'country_code'=>$request->country_code])->first();
            
            $user = User::where([
                'email' => $request->email
            ])->first();
            $credentials = request([
                'email',
                'password'
            ]);
            
            if ($user) {
                
                // checking if user is blocked.
                if ($user->enabled == 'No') {
                    return response()->json([
                        'status' => 2,
                        'message' => 'Sorry! User is Blocked.'
                    ]);
                }
                
                if (Hash::check($request->password, $user->password)) {
                    Auth::attempt($credentials);
                    $user = Auth::user();
                    
                    User::where([
                        'email' => $request->email
                    ])->update([
                        'device_token' => $request->device_token
                    ]);
                    
                    $user->auth_token = $user->createToken('AppName')->accessToken;
                    
                    return response()->json([
                        'status' => 1,
                        'message' => 'Login Successful.',
                        'data' => $user
                    ]);
                } else {
                    return response()->json([
                        'status' => 0,
                        'message' => "Incorect Password"
                    ]);
                }
            } else {
                return response()->json([
                    'status' => 0,
                    'message' => 'User Not Registered.',
                    'is_registered' => 'no'
                ]);
            }
            
        }
        
        if($data['is_social'] == 'yes')
        {
            
            if (empty($data['google_token']) && empty($data['facebook_token']) && empty($data['apple_token'])) {
                return response()->json([
                    'status' => 0,
                    'message' => "Google or Facebook or Apple Token is required"
                ]);
            }
            
            if (! empty($data['device_token'])) {
                $input['device_token'] = $data['device_token'];
            }
//             else{
//                 return response()->json([
//                     'status' => 0,
//                     'message' => "Device token is required"
//                 ]);
//             }
            
            $tokenType = null;
            if (isset($data['email']) && ! empty($data['email'])) {
                $user_check = User::where('email', $data['email'])->first();
                if ($user_check && $user_check->enabled == 'No') {
                    return response()->json([
                        'status' => 2,
                        'message' => "User with this email is Blocked."
                    ]);
                }
            }
            if (! empty($data['google_token'])) {
                $input['google_token'] = $data['google_token'];
                if (empty($user_check)) {
                    $user_check = User::where([
                        'google_token' => $data['google_token']
                    ])->first();
                }
            }
            if (! empty($data['facebook_token'])) {
                $input['facebook_token'] = $data['facebook_token'];
                if (empty($user_check)) {
                    $user_check = User::where([
                        'facebook_token' => $data['facebook_token']
                    ])->first();
                }
            }
            if (! empty($data['apple_token'])) {
                $input['apple_token'] = $data['apple_token'];
                if (empty($user_check)) {
                    $user_check = User::where([
                        'apple_token' => $data['apple_token']
                    ])->first();
                }
            }
    
            if (! empty($user_check)) {
    
                $user = User::find($user_check->id);
                User::whereId($user_check->id)->update($input);
                $record = User::find($user_check->id);
    
                $record->auth_token = $record->createToken('AppName')->accessToken;
                return response()->json([
                    'status' => 1,
                    'message' => 'Login successfull',
                    'data' => $record
                ]);
            } else {
                
                return response()->json([
                    'status' => 0,
                    'message' => 'User not Registered.',
                    'is_registered' => 'no'
                ]);
            }
        }
        return response()->json([
            'status' => 0,
            'message' => 'No data posted'
        ]);
    }

    public function register(Request $request)
    {
        $posted = $request->all();

        $posted['role'] = "Customer";
        $posted['enabled'] = 'Yes';


            $validator = Validator::make($request->all(), [
                'name' => 'required',
                'country_code' => 'required',
                'phone' => 'required|unique:users',
                'email' => 'email|unique:users|required',
//                 'password' => 'required',
                'device_token' => 'required'
            ]);
            $fields = array(
                'name',
                'phone',
                'email',
                'device_token',
                'country_code'
            );

            $error_message = "";
            if ($validator->fails()) {
                foreach ($fields as $field) {
                    if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                        $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                    }
                }
                return response()->json([
                    'status' => 0,
                    'message' => substr($error_message, 0, - 2)
                ]);
            }

            $posted['password'] = isset($posted['password']) ? bcrypt($posted['password']) : "";
            
            if ($request->hasFile('photo')) {
                $image = $request->file('photo');
                $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
                $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
                $s3 = \Storage::disk('s3');
                $filePath = '/users/' . $imageFileName;
                $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
                if ($imageResult == 1) {
                    $posted['photo'] = $imageUrl;
                }
            }

            $user = User::create($posted);
            $record = User::where([
                'country_code' => $posted['country_code'],
                'phone' => $posted['phone']
            ])->first();
            $record->auth_token = $record->createToken('AppName')->accessToken;
            return response()->json([
                'status' => 1,
                'message' => 'Registration successfull.',
                'data' => $record
            ]);
        
    }
    
    public function checkEmail(Request $request)
    {
        if(empty($request->email)){
            return response()->json([
                'status' => 0,
                'message' => "Email is required."
            ]);
        }
        $user = User::where([
            'email' => $request->email
        ])->first();
        
        if (! empty($user)) {
            return response()->json([
                'status' => 0,
                'message' => "Account with this email is already exist."
            ]);
        }
        return response()->json([
            'status' => 1,
            'message' => "Email is unique."
        ]);
    }

    public function userProfile()
    {
        $user = Auth::user();
        return response()->json([
            'status' => 1,
            'message' => 'User Profile',
            'data' => $user
        ]);
    }

    public function updateProfile(Request $request)
    {
        $user = Auth::user();

        $posted = $request->all();
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'country_code' => 'required',
            'phone' => 'required|unique:users,phone,' . $user->id,
            'email' => 'email|unique:users,email,' . $user->id
        ]);
        $fields = array(
            'name',
            'phone',
            'email',
            'country_code'
        );

        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $posted['photo'] = $imageUrl;
            }
        }

        $User = User::find($user->id);

        $User->fill($posted);

        if ($User->update()) {
            return response()->json([
                'status' => 1,
                'message' => 'User Profile updated successfully',
                'data' => $User
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => 'Not Updated.'
            ]);
        }
    }

    public function sendOtp(Request $request)
    {
        $user = User::where([
            'country_code' => $request->country_code,
            'phone' => $request['phone']
        ])->first();
        if (! empty($user)) {
            return response()->json([
                'status' => 0,
                'message' => "Account with this phone number is already exist."
            ]);
        }

        $phoneNumber = $request->country_code . $request->phone;

        // create otp from random string.
        $otp = substr(str_shuffle("0123456789"), 0, 4);
        // create chahe
        $expiresAt = Carbon::now()->addMinutes(30);
        Cache::put('phone' . $request->phone, $otp, $expiresAt);

        $bodtText = "Your login OTP for OJA : " . $otp;
        // send otp to phone number;
        $sms = User::sendSms($phoneNumber,$bodtText);
        
        if($sms['status']){
            return response()->json([
                'status' => 1,
                'message' => 'OTP sent successfully',
                'data' => $phoneNumber
            ]);
        }
        return response()->json([
            'status' => 0,
            'message' => $sms['error'],
            'data' => $phoneNumber
        ]);
    }

    public function verifyOtp(Request $request)
    {
        if(empty($request->type) && empty($request->otp) && (empty($request->phone) || empty($request->email))){
            return response()->json([
                'status' => 0,
                'message' => "No data posted."
            ]);
        }
        
        if($request->type == "phone"){
            $phoneNumber = $request->country_code . $request->phone;
            $savedOtp = Cache::get('phone' . $request->phone, '0');
    
        }else{
            $savedOtp = Cache::get('email' . $request->email, '0');
        }
        
//         $savedOtp = "1234";

        if ($savedOtp == $request->otp) {
            Cache::pull('phone' . $request->phone);
            Cache::pull('email' . $request->email);
            return response()->json([
                'status' => 1,
                'message' => 'OTP verified successfully'
            ]);
        }
        return response()->json([
            'status' => 0,
            'message' => "Invalid OTP."
        ]);
    }

    public function changePassword(Request $request)
    {
        $user = Auth::user();

        $posted = $request->all();
        $validator = Validator::make($request->all(), [
            'old_password' => 'required',
            'password' => 'required|min:6'
        ]);
        $fields = array(
            'old_password',
            'password'
        );

        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        if (Hash::check($request->old_password, $user->password)) {

            $user->password = Hash::make($posted['password']);
            if ($user->save()) {
                return response()->json([
                    'status' => 1,
                    'message' => 'Password changed successfully'
                ]);
            }
        } else {
            return response()->json([
                'status' => 0,
                'message' => "Old password not correct."
            ]);
        }

        return response()->json([
            'status' => 1,
            'message' => 'Password not changed'
        ]);
    }

    public function logout(Request $request)
    {
        $user = User::getLoginUser();
        $user->device_token = "";
        $user->save();
        
        if (Auth::check()) {
            Auth::user()->token()->revoke();
            Session::flush();
        }

        return response()->json([
            'status' => 1,
            'message' => 'Successfully logged out'
        ]);
    }
    
    public function sendOtpOnEmail(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required'
        ]);
        $fields = array(
            'email'
        );
        
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }
        
        $user = User::where([
            'email' => $request->email
        ])->first();
        if (empty($user)) {
            return response()->json([
                'status' => 0,
                'message' => "Account with this email does not exist."
            ]);
        }
        
        // create otp from random string.
        $otp = substr(str_shuffle("0123456789"), 0, 4);
        // create chahe
        $expiresAt = Carbon::now()->addMinutes(30);
        Cache::put('email' . $request->email, $otp, $expiresAt);
        
        // send otp on email
        $email_content['name'] = $user->name;
        $email_content['otp'] = $otp;
        $email_content['subject'] = "This is the OTP to reset your password. ";
        $html = view('mail.resetPassword', compact('email_content'))->render();
        $emailArr['to_email'] = $request->email;
        $emailArr['to_user_name'] =  $user->name;
        $emailArr['subject'] = "This is the OTP to reset your password.";
        User::sendEmail($emailArr, $html);
        
        return response()->json([
            'status' => 1,
            'message' => 'OTP sent successfully to this email : ' . $user->email
        ]);
    }

    public function resetPassword(Request $request)
    {
        $posted = $request->all();
        $validator = Validator::make($request->all(), [
            'password' => 'required|min:6',
            'email' => 'required'
        ]);
        $fields = array(
            'email',
            'password'
        );

        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && ! empty($validator->errors()->getMessages()[$field][0]) /*&& empty($error_message)*/) {
                    $error_message .= $validator->errors()->getMessages()[$field][0] . ", ";
                }
            }
            return response()->json([
                'status' => 0,
                'message' => substr($error_message, 0, - 2)
            ]);
        }

        $user = User::where([
            'role' => 'Customer',
            'email' => $posted['email']
        ])->first();

        if (empty($user)) {
            return response()->json([
                'status' => 0,
                'message' => "No user found with this email."
            ]);
        }

        $user->password = Hash::make($posted['password']);
        if ($user->save()) {
            return response()->json([
                'status' => 1,
                'message' => 'Password reset successfully'
            ]);
        }

        return response()->json([
            'status' => 1,
            'message' => 'Password not reset'
        ]);
    }
    
    
    //Send support us mail API
    public function sendSupportMail(Request $request)
    {
        if(empty($request->name) || empty($request->email) || empty($request->message)){
            return response()->json(['status' => 0, 'message' => "No data posted"]);
        }
        
        $email_content['email'] = $request->email;
        $email_content['name'] = $request->name;
        $email_content['message'] = $request->message;
        //Mail::to('mariam@my-oja.com')->send(new SupportMail($email_content));       
        $html = view('mail.support', compact('email_content'))->render();
        $emailArr['to_email'] = 'mariam@my-oja.com';
        $emailArr['to_user_name'] =  $request->name;       
        $emailArr['subject'] = "New suppot mail from " . $request->name;
        User::sendEmail($emailArr, $html);

        return response()->json(['status' => 1, 'message' => "Support mail successfully send to admin."]);
    }
}
