<?php
namespace App\Http\Controllers;

use App\Culture;
use Illuminate\Http\Request;
use App\Brand;
use App\Product;
use App\UserForm;

class BrandsController extends Controller
{

    public function __construct()
    {
        // $this->middleware('auth');
    }

    public function index(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['brands'] = Brand::where('name', 'like', '%' . $request->stext . '%')->paginate(50);
            $data['stext'] = $request->stext;
        } else {
            $data['brands'] = Brand::paginate(50);
            $data['stext'] = "";
        }

        return view('brands/brandsList', $data);
    }

    public function add()
    {
        return view('brands/add');
    }

    public function save(Request $request)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'name' => 'required|unique:brands'
        ], [
            'name.unique' => 'Name already exists.'
        ]);
        // print_r($posted);exit;
        $brand = new Brand();
        $brand->name = $posted['name'];
        $brand->enabled = $posted['enabled'];
        
        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;
            
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $brand->logo = $imageUrl;
            }
        }

        if ($brand->save()) {
            return redirect()->route('brandsList')->with('success', 'Brand created successfully.');
        } else {
            return back()->with('error', 'Brand has not been created.');
        }
    }

    public function edit($id)
    {
        $brand = Brand::where([
            'id' => $id
        ])->first();
        if (! $brand) {
            return redirect()->route('brandsList')->with('warning', 'Invalid Brand to edit');
        }
        return view('brands/edit', [
            'brand' => $brand
        ]);
    }
    
    //saving brand using ajax from product add page
    public function saveAjaxBrand(Request $request)
    {
        $alreadySaved = Brand::where('name',$request->brand)->first();
        
        if(!empty($alreadySaved))
        {
            return response()->json([
                'status' => 0,
                'message' => "Brand name already exist"
            ]);
        }
            
        // print_r($posted);exit;
        $brand = new Brand();
        $brand->name = $request->brand;
        $brand->enabled = 'Yes';
        
        if ($brand->save()) { 
            $brand = Brand::where('name',$request->brand)->first();
            return response()->json([
                'status' => 1,
                'brand_id' => $brand->id,
                'message' => "Brand Name Saved"
            ]);
        } else {
            return response()->json([
                'status' => 0,
                'message' => "Brand Name Not Saved"
            ]);
        }
    }

    public function update(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'name' => 'unique:brands,id,' . $id
        ], [
            'name.unique' => 'Name already exists.'
        ]);

        $brand = Brand::where([
            'id' => $id
        ])->first();
        if (! $brand) {
            return redirect()->route('brandsList')->with('warning', 'Invalid brand to edit');
        }
        $brand->name = $posted['name'];
        $brand->enabled = $posted['enabled'];
        
        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;
            
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $brand->logo = $imageUrl;
            }
        }

        if ($brand->save()) {
            return redirect()->route('brandsList')->with('success', 'Brand updated successfully.');
        } else {
            return back()->with('error', 'Brand has not been modified.');
        }
    }

    public function delete(Request $request)
    {
        if (Brand::find($request->id)->forceDelete()) {

            return redirect()->back()->with('success', 'Brand has been Deleted!');
        } else {
            return redirect()->back()->with('error', 'Brand has not been Deleted!');
        }
    }

    /**
     * API start here
     */
    public function getBrandsList()
    {        
        $data['brands'] = Brand::where('enabled', 'Yes')->paginate(30);

        if (count($data['brands']) == 0) {
            return response()->json([
                'status' => 0,
                'message' => "Brands not found."
            ]);
        }

        return response()->json([
            'status' => 1,
            'message' => "Brand Send Successfully",
            'data' => $data
        ]);
    }
    
    public function getBrandProducts(Request $request)
    {
        if(empty($request->brand_id)){
            return response()->json([
                'status' => 0,
                'message' => "Brand id is required."
            ]);
        }
        
        $brand = Brand::where(['enabled'=>'Yes','id'=>$request->brand_id])->first();
        
        if(empty($brand)){
            return response()->json([
                'status' => 0,
                'message' => "Brand not found."
            ]);
        }
        
        if(isset($request->seller_id))
        {
            $sellerId = $request->seller_id;
            
            $data['products'] = Product::with(['brand','variations.currentBatch','variations.images','variations.cart','variations.wishList'])
                                ->with(['variations'=>function($query1) use ($sellerId){
                                    $query1->whereHas('currentBatch',function($query2) use ($sellerId){
                                        $query2->where('seller_id', $sellerId);
                                        $query2->whereHas('users',function($query3){
                                            $query3->where('store_exist','Yes');
                                        });
                                    });
                                }])
                                ->whereHas('variations',function($query) use ($sellerId){
                                    $query->whereHas('currentBatch',function($query1) use ($sellerId){
                                        $query1->where('seller_id', $sellerId);
                                        $query1->whereHas('users',function($query2){
                                            $query2->where('store_exist','Yes');
                                        });
                                    });
                                })
                                ->where('brand_id', $brand->id)
                                ->where('enabled','Yes')->paginate(10);
        }
        else 
        {
            $data['products'] = Product::with(['brand','variations.currentBatch','variations.images','variations.cart','variations.wishList'])
                                ->with(['variations'=>function($query1){
                                    $query1->whereHas('currentBatch',function($query2){
                                        $query2->whereHas('users',function($query3){
                                            $query3->where('store_exist','Yes');
                                        });
                                    });
                                }])
                                ->whereHas('variations',function($query){
                                    $query->whereHas('currentBatch',function($query1){
                                        $query1->whereHas('users',function($query2){
                                            $query2->where('store_exist','Yes');
                                        });
                                    });
                                })
                                ->where('brand_id', $brand->id)
                                ->where('enabled','Yes')->paginate(10);
        }
                            
        return response()->json([
            'status' => 1,
            'message' => "Brand Send Successfully",
            'data' => $data
        ]);
    }
    
    public function importBrands()
    {
        return view('brands/import');
    }
    
    public function saveImportBrands(Request $request)
    {
        $fp = file($request->csv_file);
        //echo count($fp);die();
        if(count($fp) > 10000){
            return redirect()->route('brandsList')->with('error', 'Can not import more than 10,000 brands in one file.');
        }
        
        $path = $request->csv_file->getRealPath();
        $data = array_map('str_getcsv', file($path));
        
        $brands = array_slice($data, 0);
        
//         foreach($brands as $key => $brand)
//         {
//             if($key == 0){
//                 continue;
//             }
//             $newBrand = $brand[1];
//             $old = Brand::where('name',$newBrand)->first();
//             if(empty($old)){
//                 $brandObj = new Brand();
//                 $brandObj->name = $newBrand;
//                 $brandObj->save();
//             }
            
//         }


        foreach($brands as $key => $brand)
        {
           
            if($key == 0){
                continue;
            }
            echo "<pre>";print_r($brands);exit;
            
            //code for adding join our community data
//            $userForm = new UserForm();
//            $userForm->email = $brand[0];
//            $userForm->postcode = $brand[1];
//            $userForm->cultures = $brand[2];
//            $userForm->type = UserForm::FORM_TYPE_JOIN_COMMUNITY;
//            $userForm->save();

//            code for adding get your store data
//            $userForm = new UserForm();
//            $userForm->full_name = $brand[0];
//            $userForm->email = $brand[1];
//            $userForm->postcode = $brand[2];
//            $userForm->company_name = $brand[3];
//            $userForm->cultures = $brand[4];
//            $userForm->current_services = $brand[5];
//            $userForm->type = UserForm::FORM_TYPE_GET_YOUR_STORE;
//            $userForm->save();

            
        }


        return redirect()->route('brandsList')->with('success', 'Brands import successfully.');
    }
    
    public function exportSampleBrands()
    {
        $fileName = 'sample-brands.csv';
        
        $headers = array(
            "Content-type"        => "text/csv",
            "Content-Disposition" => "attachment; filename=$fileName",
            "Pragma"              => "no-cache",
            "Cache-Control"       => "must-revalidate, post-check=0, pre-check=0",
            "Expires"             => "0"
        );
        
        $columns = array('Sr. No.', 'Brands');
        
        $callback = function() use($columns) {
            $file = fopen('php://output', 'w');
            fputcsv($file, $columns);
            
            $row['Sr. No.']  = "1";
            $row['Brands'] = "Amul";
            fputcsv($file, array($row['Sr. No.'], $row['Brands']));
            
            fclose($file);
        };
        return response()->stream($callback, 200, $headers);
    }
}
