<?php
namespace App\Http\Controllers;

ini_set('memory_limit', '-1');
ini_set('upload_max_filesize', '1024M');

use App\Category;
use App\Product;
use App\Section as AppDashboard;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Image;
use App\ProductImage;
use Tinify\AccountException;

class CategoriesController extends Controller
{

    public function __construct()
    {
        // $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
        $categories = Category::orderBy('sort_order', 'asc')->orderBy('parent_id', 'asc')
            ->orderBy('name', 'asc')
            ->with('parent');

        $posted = $request->all();
        if (! empty($posted)) {
            if (! empty($posted['search'])) {
                $categories->where('name', 'like', '%' . $posted['search'] . '%');
            }
        }
        $categories = $categories->paginate(75);

        return view('categories/index', [
            'categories' => $categories,
            'posted' => $posted
        ]);
    }

    public function create()
    {
        /*
         * $catTree = Category::whereNull('parent_id')->with(['categories'=>function($query){
         * $query->where('enabled','Yes');
         * }])->get()->toArray();
         *
         * $catTree = $this->genCatList($catTree);
         *
         */
        $catTree = Category::whereNull('parent_id')->orderBy('sort_order', 'asc')
            ->get()
            ->toArray();

        return view('categories/create', [
            'parents' => $catTree
        ]);
    }

    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required',
            'image' => 'image'
        ], [
            'name.required' => "Category Name is required"
        ]);

        $category = new Category();

        $category->name = $request->name;
        $category->parent_id = empty($request->parent_id) ? null : $request->parent_id;
        $category->enabled = $request->enabled;

        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;

//             Compress image using tinify
            \Tinify\setKey("270QBN29NtRsKxFWHzb5YdYtsfXLbnCy");
            $sourceData = file_get_contents($image);
            $resultData = \Tinify\fromBuffer($sourceData)->toBuffer();
            
//           Saving Compressed image to AWS
            $imageResult = $s3->put($filePath, $resultData, 'public');
            
            if ($imageResult == 1) {
                $category->image = $imageUrl;
            }
        }

        if ($category->save()) {
            return redirect('/categories')->with('success', 'New Category created successfully!');
        } else {
            return redirect('/categories')->with('warning', 'New Category has not been created!');
        }
    }
    
    public function compressImage()
    {
        \Tinify\setKey("cvmxtjqYtBRZSbNvDcp2w8zdkDbdg1TW");
        
        $productImages = ProductImage::where('compress',null)->get();
        
//         print_r(count($productImages));exit;
        
        if(count($productImages) > 0){
            
            try {
                
                foreach ($productImages as $productImage){
                    
                    $ext = pathinfo(
                        parse_url($productImage->name, PHP_URL_PATH),
                        PATHINFO_EXTENSION
                        );
                    
                    if(empty($ext) || $ext != "JPG"){
                        continue;
                    }
                    
                    $sourceData = file_get_contents($productImage->name);
                    $resultData = \Tinify\fromBuffer($sourceData)->toBuffer();
                    
                    $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $ext;
                    $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/products/" . $imageFileName;
                    $s3 = \Storage::disk('s3');
                    $filePath = '/products/' . $imageFileName;
                    $imageResult = $s3->put($filePath, $resultData, 'public');
                    
                    if ($imageResult == 1) {
                        $productImage->name = $imageUrl;
                        $productImage->compress = "Yes";
                        $productImage->save();
                        echo "<br>". $productImage->id . " - Compressed </br>";
                    }else{
                        echo "<br>". $productImage->id . " - Not Compressed </br>";
                    }
                    
                }
                
            } catch (AccountException $e) {
                print_r($e->getMessage());exit;
            }
            
            echo "Task Complete.";
        }else{
            echo "No Image Found.";
        }
         
    }

    public function show($id)
    {
        return view('categories/show');
    }

    public function edit($id)
    {
        $category = Category::findOrFail($id);

        /*
         * $catTree = Category::whereNull('parent_id')->with(['categories'=>function($query){
         * $query->where('enabled','Yes');
         * }])->get()->toArray();
         *
         * $catTree = $this->genCatList($catTree);
         */
        $catTree = Category::whereNull('parent_id')->where('id', '!=', $id)
            ->where('enabled', 'Yes')
            ->get()
            ->toArray();

        return view('categories/edit', [
            'category' => $category,
            'parents' => $catTree
        ]);
    }

    public function update(Request $request, $id)
    {
        $request->validate([
            'name' => 'required',
            'image' => 'image'
        ], [
            'name.required' => "Category Name is required"
        ]);

        $category = Category::find($id);

        $category->name = $request->name;
        $category->parent_id = empty($request->parent_id) ? null : $request->parent_id;
        $category->enabled = $request->enabled;

        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $category->image = $imageUrl;
            }
        }

        if ($category->save()) {
            return redirect('/categories')->with('success', 'Category updated successfully!');
        } else {
            return redirect('/categories')->with('warning', 'Category has not been updated!');
        }
    }

    public function destroy($id)
    {
        $catTree = Category::where('id', $id)->with('categories')
            ->get()
            ->toArray();
        $catTree = $this->genCatChildrenIdsList($catTree);

        // initiallization of DB transactions
        DB::beginTransaction();

        try {
            // getting uncategorized category
            $uncat = Category::whereNotNull('parent_id')->whereName('Uncategorized')->first();
            // getting delete category products
            $products = Product::with('categories')->whereHas('categories', function ($query) use ($catTree) {
                $query->whereIn('category_id', $catTree);
            })
                ->get();

            // updating product categories
            foreach ($products as $product) {
                $product->categories()->detach($catTree);
                if (count($product->categories) == 1) {
                    $product->categories()->attach($uncat->id);
                }
            }
            // deleting the categories
            Category::destroy($catTree);

            // DB::rollback();
            DB::commit();

            return redirect('/categories')->with('success', 'Category deleted successfully!');
        } catch (\Exception $e) {
            DB::rollback();
            return redirect('/categories')->with('warning', 'Category has not been deleted!');
        }

        return redirect('/categories');
    }

    public function appDashboardSettings(Request $request)
    {
        if ($request->isMethod('POST')) {
            $posted = collect($request->all())->toArray();

            if ($posted['type'] == 'sections') {

                $flag = 0;
                foreach ($posted['data'] as $key => $secId) {
                    $section = AppDashboard::find($secId);
                    $section->sort_order = $key;
                    $section->save();
                    $flag ++;
                }
                if ($flag == count($posted['data'])) {
                    echo 1;
                } else {
                    echo 0;
                }
                return;
            } else if ($posted['type'] == 'categories' || $posted['type'] == 'subcategories') {
                $flag = 0;

                foreach ($posted['data'] as $key => $secId) {
                    // $section = Category::find($secId);
                    // $section->sort_order = $key;
                    // $section->save();

                    $ustat = Category::where('id', $secId)->update([
                        'sort_order' => $key
                    ]);

                    if ($ustat) {
                        $flag ++;
                    }
                }

                if ($flag == count($posted['data'])) {
                    echo 1;
                } else {
                    echo 0;
                }

                return;
            } else {
                echo 0;
                return;
            }
        }

        $data['categories'] = Category::where('name', '!=', 'Uncategorized')->where('enabled', 'Yes')
            ->where('parent_id', '')
            ->orWhere('parent_id', '0')
            ->orWhereNull('parent_id')
            ->with([
            'categories' => function ($query) {
                $query->where('enabled', 'Yes');
            }
        ])
            ->orderBy('sort_order', 'asc')
            ->get();

        $data['sections'] = AppDashboard::orderBy('sort_order', 'asc')->get();
        return view('categories/appDashboardSettings', $data);
    }

    public function genCatList($tree, $saperator = "", $cTree = [])
    {
        foreach ($tree as $k => $row) {
            $cTree[] = array(
                'id' => $row['id'],
                'name' => $saperator . $row['name']
            );
            $cTree = $this->genCatList($row['categories'], $saperator . $row['name'] . " &mdash;> ", $cTree);
        }

        return $cTree;
    }

    public function genCatChildrenIdsList($tree, $saperator = "", $cTree = [])
    {
        foreach ($tree as $k => $row) {
            $cTree[] = $row['id'];
            $cTree = $this->genCatChildrenIdsList($row['categories'], $saperator . $row['name'] . " &mdash;> ", $cTree);
        }

        return $cTree;
    }

    /* API Methods */
    public function subCategoriesOfCategory($id)
    {
        DB::enableQueryLog();
        $data['category'] = Category::with([
            'subCategories' => function ($query) {
                $query->where('enabled', 'Yes');
            }
        ])->where('id', $id)
            ->withCount([
            'products' => function ($query) {
                $query->where('enabled', 'Yes');
            }
        ])
            ->get();
        $query = DB::getQueryLog();
        // dd($query);
        return response()->json([
            'status' => 1,
            'message' => 'Sub Categories of Category',
            'data' => $data
        ]);
    }
    /* API Methods ends here. */
}
