<?php

namespace App\Http\Controllers;
use App\Merchant;
use App\Product;
use App\Sku;
use Illuminate\Http\Request;
use Validator;
use DB;

class MerchantsController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
    	$posted = $request->all();
		
		if(!empty($posted)){
			$field = $posted['field'];
			$value = $posted['value'];
			
			$merchants = Merchant::where($field,'like','%'.$value.'%');
			if(!empty($posted['enabled'])){
				$merchants->where('enabled',$posted['enabled']);
			}
			
			$merchants = $merchants->paginate(75);
			
		} else {
			$merchants = Merchant::paginate(75);
		}
    		
		
        return view('merchants/index',['merchants'=>$merchants,'posted'=>$posted]);
    }
	
	
	
	public function show($id)
    {
    	$merchant = Merchant::with(['products.categories'])->find($id);
		/*
		$skus = Sku::with(['product.categories','product.tax','images'])
				->whereHas('batches',function($q)use($id){
					$q->whereHas('invoice',function($qq)use($id){
						$qq->where('merchant_id',$id);
					});
				})->paginate(20);
		*/
		
		
		//dd($skus->toArray());
			
        return view('merchants/show',['merchant'=>$merchant/*,'stats'=>$stats*/]);
    }
	
	public function create()
    {
    	return view('merchants/create');
    }
	
	public function getMerchantProducts($mid){
		//$products=Merchant::where('id',$mid)->first()->products()->with('categories')->paginate(20);
		$skus = Sku::with(['product.categories','product.tax','images'])
				->whereHas('batches',function($q)use($mid){
					$q->whereHas('invoice',function($qq)use($mid){
						$qq->where('merchant_id',$mid);
					});
				})->paginate(20);
        return view('merchants/getMerchantProducts',['skus'=>$skus]);
	}
	public function getMerchantSales($mid){
		$sales=Merchant::where('id',$mid)->first()->order_products()->with('order.user')->paginate(20);
		
        return view('merchants/getMerchantSales',['sales'=>$sales]);
	}
	
	public function store(Request $request){
    	
		$request->validate([
            'name' => 'required',
            'merchant_code' => 'required',
            'email' => 'required|unique:merchants',
            'phone' => 'required|unique:merchants',
            'contact_name' => 'required',
            'password' => 'required|regex:/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-8])(?=.*?[#?!@$%^&*-]).{6,}$/',
            'logo' => 'image'
            
        ],[
        	'name.required'=>"Merchant Name is required",
        	'email.required'=>"Merchant Email is required",
        	'email.unique'=>"Merchant with this email is already registered",
        	'phone.required'=>"Merchant Phone is required",
        	'phone.unique'=>"Merchant with this Phone is already registered",
        	'contact_name.required' => 'Contact Person Name is required',
            'password.required' => 'Login Password is required',
            'password.regex' => 'Password must be more than 8 characters long, should contain at-least 1 Uppercase, 1 Lowercase, 1 Numeric and 1 special character',
            'logo.image' => "Only Images are supported as Logo"
        ]);
		
		
        $mdata = Collect($request->all())->forget(['logo','_token'])->put('password',bcrypt($request->password))->toArray();
		if($request->hasFile('logo')){
			$image = $request->file('logo');
	        $imageFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $image->getClientOriginalExtension();
	        $imageUrl="https://gadgetzone.s3-ap-south-1.amazonaws.com/images/merchants/".$imageFileName;
	        $s3 = \Storage::disk('s3');
			$filePath = '/images/merchants/' . $imageFileName;
	        $imageResult=$s3->put($filePath, file_get_contents($image), 'public');
			if($imageResult==1){
				$mdata['logo'] = $imageUrl;
			} 
		}
		
		$merchant = new Merchant;
		$merchant->fill($mdata);
		
		if($merchant->save()){
			return redirect('/merchants')->with('success','New Merchant created successfully!');
		} else {
			return redirect('/merchants')->with('warning','New Merchant has not been created!');
		}
    }


	public function edit($id)
    {
    	$merchant = Merchant::find($id);
		
    	return view('merchants/edit',['merchant'=>$merchant]);
    }
	
	public function update(Request $request, $id){
    	
		
		
		if(empty($request->password)){
			$request->validate([
	            'name' => 'required',
	            'merchant_code' => 'required',
	            'email' => 'required|unique:merchants,email,'.$id,
	            'phone' => 'required|unique:merchants,phone,'.$id,
	            'contact_name' => 'required',
	            //'password' => 'regex:/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-8])(?=.*?[#?!@$%^&*-]).{6,}$/',
	            'logo' => 'image'
	            
	        ],[
	        	'name.required'=>"Merchant Name is required",
	        	'email.required'=>"Merchant Email is required",
	        	'email.unique'=>"Merchant with this email is already registered",
	        	'phone.required'=>"Merchant Phone is required",
	        	'phone.unique'=>"Merchant with this Phone is already registered",
	        	'contact_name.required' => 'Contact Person Name is required',
	            //'password.required' => 'Login Password is required',
	            //'password.regex' => 'Password must be more than 8 characters long, should contain at-least 1 Uppercase, 1 Lowercase, 1 Numeric and 1 special character',
	            'logo.image' => "Only Images are supported as Logo"
	        ]);
			$mdata = Collect($request->all())->forget(['logo','_token'])->toArray();
		} else {
			$request->validate([
	            'name' => 'required',
	            'merchant_code' => 'required',
	            'email' => 'required|unique:merchants,email,'.$id,
	            'phone' => 'required|unique:merchants,phone,'.$id,
	            'contact_name' => 'required',
	            'password' => 'regex:/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-8])(?=.*?[#?!@$%^&*-]).{6,}$/',
	            'logo' => 'image'
	            
	        ],[
	        	'name.required'=>"Merchant Name is required",
	        	'email.required'=>"Merchant Email is required",
	        	'email.unique'=>"Merchant with this email is already registered",
	        	'phone.required'=>"Merchant Phone is required",
	        	'phone.unique'=>"Merchant with this Phone is already registered",
	        	'contact_name.required' => 'Contact Person Name is required',
	            //'password.required' => 'Login Password is required',
	            'password.regex' => 'Password must be more than 8 characters long, should contain at-least 1 Uppercase, 1 Lowercase, 1 Numeric and 1 special character',
	            'logo.image' => "Only Images are supported as Logo"
	        ]);
			$mdata = Collect($request->all())->forget(['logo','_token'])->put('password',bcrypt($request->password))->toArray();
		}
        
		if($request->hasFile('logo')){
			$image = $request->file('logo');
	        $imageFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $image->getClientOriginalExtension();
	        $imageUrl="https://gadgetzone.s3-ap-south-1.amazonaws.com/images/merchants/".$imageFileName;
	        $s3 = \Storage::disk('s3');
			$filePath = '/images/merchants/' . $imageFileName;
	        $imageResult=$s3->put($filePath, file_get_contents($image), 'public');
			if($imageResult==1){
				$mdata['logo'] = $imageUrl;
			} 
		}
		
		$merchant = Merchant::find($id);
		$merchant->fill($mdata);
		
		if($merchant->update()){
			return redirect('/merchants')->with('success','Merchant Updated successfully!');
		} else {
			return redirect('/merchants')->with('warning','Merchant has not been updated!');
		}
    }


	public function destroy($id){
		
		DB::beginTransaction();
		
		try {
			Product::where('merchant_id',$id)->delete();
			Merchant::destroy($id);
			
			DB::commit();
			return redirect()->back()->with('success','Merchant has been Deleted!');
		} catch (\Exception $e) {
			DB::rollback();
			return redirect()->back()->with('error','Merchant has not been Deleted!');
		}
		
	}
	
	
	
}
