<?php

namespace App\Http\Controllers;
use App\Offer;
use Illuminate\Http\Request;
use Validator;
use DB;
use Log;


class OffersController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    	
	public function index(Request $request)
	{
		
		$curl = curl_init();
		curl_setopt_array($curl, array(
		  CURLOPT_URL => env('curl_url').'specialOffers',
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_ENCODING => '',
		  CURLOPT_MAXREDIRS => 10,
		  CURLOPT_TIMEOUT => 0,
		  CURLOPT_FOLLOWLOCATION => true,
		  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		  CURLOPT_CUSTOMREQUEST => 'GET',
		));

		$response = curl_exec($curl);
		curl_close($curl);
		
		$data['offers'] = json_decode($response);
		$data['ftype'] = "";
		$data['stext'] = "";
    	return view('offers/index', $data);

	}

	public function create()
    {
    	
        return view('offers/create');
    }
	
	public function store(Request $request){
    	
		$request->validate([
            'name' => 'required',		 
		    'code' => 'required',
        ],[
        	'name.required'=>"Offer Title is required",           
            'code.required'=>"Coupon code is required"
        ]);

		try {
			 $curl = curl_init();

			  curl_setopt_array($curl, array(
			  CURLOPT_URL => env('curl_url').'specialOffers',
			  CURLOPT_RETURNTRANSFER => true,
			  CURLOPT_ENCODING => '',
			  CURLOPT_MAXREDIRS => 10,
			  CURLOPT_TIMEOUT => 0,
			  CURLOPT_FOLLOWLOCATION => true,
			  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			  CURLOPT_CUSTOMREQUEST => 'POST',
			  CURLOPT_POSTFIELDS =>'{
			 		"offerName":"'.$request->name.'",
					 "discountType":"'.$request->discount_type.'",
					 "discountAmount":"'.$request->amount.'",
					 "discountPercent": "'.$request->discount_upto.'",
					 "offerType": "'.$request->offer_type.'",
					 "promoCodeCount": "'.$request->promocodecount.'",
					 "minimumSpend": "'.$request->min_amount.'",
					 "maximumDiscount": "'.$request->amount.'",					
					 "triggerTypeId":1,
					 "startDateTime": "'.$request->start_date.'T00:00:00'.'",
					 "endDateTime": "'.$request->end_date.'T00:00:00'.'",
					 "isEnabled": "'.$request->enabled.'",
					 "discountCode": "'.$request->code.'",
					 "userId":12
			 	}',	
			  CURLOPT_HTTPHEADER => array(
			    'Content-Type: application/json'
			  ),
			));			

			$response = curl_exec($curl);
			if (curl_errno($curl)) {
			    $error_msg = curl_error($curl);				    
			    Log::error($error_msg);
				return redirect()->back()->with('error','Something went wrong!');         							
			}
			curl_close($curl);

			return redirect('/offers')->with('success','New Offer created successfully!');
		} catch (\Exception $e) {
			$exception = $e->getMessage();
			Log::error($exception);
			return redirect('/offers')->with('error','New Offer has not been created!');
		}		
    }
	
	public function edit($id)
    {
    	$offer = Offer::find($id);
		
        return view('offers/edit',['offer'=>$offer]);			
        
    }
	
	public function update(Request $request,$id){
    	
	    $request->validate([
	        'name' => 'required',
	        'amount' => 'required',
	        'code' => 'required',
	        //             'image'=>'required|mimes:jpeg,bmp,png,jpg,svg'
	    ],[
	        'name.required'=>"Offer Title is required",
	        'amount.required'=>"Amount is required",
	        'code.required'=>"Coupon code is required",
	        //         	'image.required'=>"Image is required",
	    //         	'image.mimes'=>'Only Images are allowed.'
	        
	    ]);
		
        $offer = Offer::find($id);
		
        $offer->name = $request->name;
        $offer->amount = $request->amount;
        $offer->discount_type = $request->discount_type;
        $offer->min_amount = $request->min_amount ?? '0';
        $offer->discount_upto = $request->discount_upto;
        $offer->start_date = $request->start_date;
        $offer->end_date = $request->end_date;
        $offer->code = $request->code;
        $offer->enabled = $request->enabled;
		
		if($offer->save()){
			return redirect('/offers')->with('success','Offer updated successfully!');
		} else {
			return redirect('/offers')->with('error','Offer has not been updated!');
		}
    }

	public function destroy(Request $request,$id){
		//this will soft delete the product
		
		if(Offer::destroy($id)){
			return redirect('/offers')->with('success','Offer has been Deleted!');
		} else {
			return redirect('/offers')->with('error','Offer has not been Deleted!');
		}
	}
	
	//ajax to get new SKU No
	public function getNewCouponCode(Request $request){
	    echo $this->genCouponCode();
	}
	
	//generate new sku no
	public function genCouponCode(){
	    $cc = "OJA".rand(1111,9999);
	    $offer = Offer::where('code',$cc)->first();
	    if(!empty($offer)){
	        genCouponCode();
	    }
        return $cc;
	        
	}
	
	/*
	 * Api Start here
	 */
	
	public function getOffersList()
	{
	    $offers = Offer::where('enabled','Yes')->where('end_date', '>=', date('Y-m-d'))->paginate(20);
	    
	    return response()->json(['status' => 1, 'message' => 'Offers sent successfully', 'data' => $offers]);
	}
	
	
}
