<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Order;
use App\OrderProduct;
use App\OrderProductStatus;
use App\Address;
use App\Product;
use App\Sku;
use DB;
use App\Cart;
use App\Batch;
use App\User;
use Carbon\Carbon;

use OneSignal; //notifications to mobile phone

use Stripe\Stripe;
use Stripe\Charge;
use Illuminate\Support\Facades\Auth;
use App\TimeSlot;
use App\OrderStatus;
use Illuminate\Support\Facades\Validator;
use App\Offer;
use Illuminate\Support\Facades\Mail;
use App\Mail\SupportMail;
use App\Mail\OrderMail;

class OrdersController extends Controller
{
	public $humanOrderStatuses = [
    		'Pending'=> "Pending to Acknowledge",
    		'Acknowledged'=> "Acknowledged",
    	    "Processing"=>"in Processing",
    	    "Ready to Pickup"=>"Ready to Pickup",
    		"Out for Delivery"=>"Out for Delivery",
    		"Rejected"=>"Rejected",
    		"Delivered"=>"Delivered",
//     		'Shipped'=>"Shipped"
	    
    	];
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
	public function index(Request $request){

		
        if(User::isSeller()){
            $orderIds = OrderProduct::select('order_id')
                        ->where('seller_id' , Auth::user()->id)
						->distinct()->pluck('order_id')->toArray();
						
           
            $orders = Order::whereIn('id', $orderIds)
                    ->with('user')->with('order_products')->orderBy('created_at','desc');
            
            $earning = Order::whereIn('id', $orderIds)->where('payment_status',"Done")->sum('total_amount');
            $today = Order::whereIn('id', $orderIds)->whereDate('created_at',date('Y-m-d'))->count();
			$total = Order::whereIn('id', $orderIds)->where('status',"Pending")->count();
			if(isset($request->today_order)){
				$orders = Order::whereIn('id', $orderIds)
				->whereDate('created_at', '=', date('Y-m-d'))
				->with('user')->with('order_products')->orderBy('created_at','desc');

			}

			if(isset($request->Tomorrow_order)){
				$orders = Order::whereIn('id', $orderIds)
				->whereDate('created_at', '=', Carbon::tomorrow()->toDateString())
				->with('user')->with('order_products')->orderBy('created_at','desc');

			}

			if(isset($request->Week_order)){
				$current = Carbon::today();
				$orders = Order::whereIn('id', $orderIds)
				->whereBetween('created_at', [$current->startOfWeek()->toDateString(),$current->endOfWeek()->toDateString()])
				->with('user')->with('order_products')->orderBy('created_at','desc');

			}
            
        }else{
            $orders = Order::with('user')->with('order_products')->orderBy('id','desc');
            
            $earning = Order::where('payment_status',"Done")->sum('total_amount');
            $today = Order::whereDate('created_at',date('Y-m-d'))->where('payment_status','Done')->count();
            $total = Order::where('payment_status',"Done")->count();
        }
    	     
		$posted = [];
		if(!empty($request->all())){
			$posted = $request->all();
			//DB::enableQueryLog();
			if(!empty($posted['order_number'])){
				$orders = $orders->where('order_number','like','%'.$posted['order_number'].'%');
			}
			if(!empty($posted['user_name'])){
				$orders = $orders->whereHas('user',function($query)use($posted){
					$query->where('name','like','%'.$posted['user_name'].'%');
				});
			}
			if(!empty($posted['order_status'])){
				//print_r($posted['order_status']);exit;
				$orders = $orders->where('status','like','%'.$posted['order_status'].'%');
				
				// $orders = $orders->whereHas('order_products',function($query)use($posted){
				// 	$query->where('status','like','%'.$posted['order_status'].'%');
				// });
			}
			if(!empty($posted['price_range'])){
				$prange = explode(';',$posted['price_range']);
				$orders = $orders->whereBetween('total_amount',$prange);
			}
			
			
			if(!empty($posted['dage_range'])){
				$range = explode(" to ",$posted['date_range']);
				$range[0] = date('Y-m-d',strtotime($range[0]));
				$range[1] = date('Y-m-d',strtotime($range[1]));
				$orders = $orders->whereBetween('created_at',$range);
			}
			if(!empty($posted['payment_mode'])){
				$orders = $orders->where('payment_type','like','%'.$posted['payment_mode'].'%');
			}
		}
    	$orders = $orders->paginate(10);	
		$minPrice = Order::/*where('payment_status',"Done")->*/min('total_amount');
		$maxPrice = Order::/*where('payment_status',"Done")->*/max('total_amount');
		
		if(User::isSeller())
		{
			//print_r($posted['order_status']);exit;
		    return view('seller/orders/index',[
		        'orders'=>$orders,
		        'query'=>$posted,
		        'minPrice'=>$minPrice,
		        'maxPrice'=>$maxPrice,
		        'earning'=>$earning,
		        'today'=>$today,
		        'total'=>$total
		    ]);
		}
        return view('orders/index',[
            'orders'=>$orders,
            'query'=>$posted,
            'minPrice'=>$minPrice,
            'maxPrice'=>$maxPrice,
            'earning'=>$earning,
            'today'=>$today,
            'total'=>$total
        ]);
    }
	
	public function show($id)
    {
    	$order = Order::where(['id'=>$id])->with('user')->with('order_products.order_status')->first();
    //	dd($order);
		if(empty($order)){
			return redirect()->back()->with('error','Invalid Order Information.');
		}
		if(User::isSeller()){
		    return view('seller/orders/order-detail',['order'=>$order]);
		}
        return view('orders/show',['order'=>$order]);
    }
    
    
    public function updateOrderStatus(Request $request){
        
        $posted = $request->all();
        
        //getting product information
        $order = Order::where('id',$posted['id'])->first();
        
        if($order){
           // $OstatusData for Order table
           // $OSstatusData for OrderStatus
           
            if($posted['status']=="Delivered"){
                $OstatusData=[
                    'status'=>$posted['status'],
                    'received_by'=>$posted['received_by']
                ];
                $OSstatusData=[
                    'order_id'=>$posted['id'],
                    'status'=>$posted['status'],
                    'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
                    'remarks'=>$posted['remarks']
                ];
            } elseif($posted['status']=="Shipped"){
                $OstatusData=[
                    'status'=>$posted['status'],
                    'awb'=>$posted['awb_number']
                ];
                $OSstatusData=[
                    'order_id'=>$posted['id'],
                    'status'=>$posted['status'],
                    'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
                    'remarks'=>$posted['remarks'],
                    'shipping'=>"Agency: ".$posted['shipping_agency'].", AWB: ".$posted['awb_number']
                ];
            } else {
                $OstatusData=[
                    'status'=>$posted['status']
                ];
                $OSstatusData=[
                    'order_id'=>$posted['id'],
                    'status'=>$posted['status'],
                    'status_date'=>date('Y-m-d H:i:s'),
                    'remarks'=>$posted['remarks']
                ];
            }
            
//             print_r($OstatusData);exit;
            
            if(Order::where('id',$posted['id'])->update($OstatusData)){
                
                $orderStatus = new OrderStatus();
                $orderStatus->fill($OSstatusData);
                $orderStatus->save();
                
                //Creating title dynamically based on status
                $notiTitle = "";
                $storeName = ($order->order_products[0]->user) ? $order->order_products[0]->user->name : "Store";
                if($posted['status'] == Order::STATE_CONFIRMED)
                {
                    $notiTitle = "Your order is confirmed by " . $storeName;
                }
                else if ($posted['status'] == "Processing")
                {
                    $notiTitle = "Your order is being prepared by " . $storeName;
                }
                else if ($posted['status'] == "Out for Delivery")
                {
                    $notiTitle = "Your order is out for delivery.";
                }
                else if ($posted['status'] == "Delivered")
                {
                    $notiTitle = "Your order has been delivered successfully";
                }
                else if ($posted['status'] == "Rejected")
                {
                    $notiTitle = $storeName . " cannot accept your order right now.";
                }
                else 
                {
                    $notiTitle = $posted['status'];
                }
                
                //Sending push notification 
                $user = User::find($order->user_id);
                if($user->notifications == 'Yes' && !empty($user->device_token))
                {
                    $title = $notiTitle;
                    $data = ['notification_type'=>'track','id'=>$order->id,'orderStatus'=>$posted['status']];
                    $response = $user->sendNotification($user->device_token,$title,$data);
                }
                
                //Sending mail if order is rejected
                $userEmail = $user->email;
                if(!empty($userEmail) && $posted['status'] == "Rejected")
                {
                    $email_content['order'] = $order;
                    $html = view('mail.rejectOrderDetails', compact('email_content'))->render();
                    $emailArr['to_email'] = $userEmail;
                    $emailArr['to_user_name'] =  $user->name;
                    $emailArr['subject'] = "Your order has not been rejected.";
                    User::sendEmail($emailArr, $html);
                }
                
                if(!empty($response) && $response->recipients){
                    $msg = "Order status updated and notification send successfully";
                    $msgType = "success";
                }else{
                    $msg = "Order status updated successfully, but error in notification sending.";
                    $msgType = "error";
                }
                return redirect()->back()->with($msgType,$msg);
                
            } else {
                return redirect()->back()->with('error',"Sorry! Not Updated. Try again.");
            }
            
            
            
            
        } else {
            return redirect()->back()->with('error',"Invalid Credentials. Try again.");
        }
        
        
    }
	
	
// 	public function updateOrderProductStatus(Request $request){
// 		$posted = $request->all();
		
// 		//getting product information
// 		$orderProduct = OrderProduct::where('id',$posted['id'])->first();
		
		
// 		if($orderProduct){
// 			if($posted['status']=="Delivered"){
// 				$ostatusData=[
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
// 					'status_remarks'=>$posted['remarks'],
// 					'received_by'=>$posted['received_by']
// 				];
// 				$statusData=[
// 					'order_product_id'=>$posted['id'],
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
// 					'remarks'=>$posted['remarks']
// 				];
// 				$noti=[
// 					'title'=>'Order'
// 				];
// 			} elseif($posted['status']=="Shipped"){
// 				$ostatusData=[
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
// 					'status_remarks'=>$posted['remarks'],
// 					'awb'=>$posted['awb_number'],
// 					'shipping_agency'=>$posted['shipping_agency']
// 				];
// 				$statusData=[
// 					'order_product_id'=>$posted['id'],
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s',strtotime($posted['status_date'])),
// 					'remarks'=>$posted['remarks'],
// 					'shipping'=>"Agency: ".$posted['shipping_agency'].", AWB: ".$posted['awb_number']
// 				];
// 			} else {
// 				$ostatusData=[
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s'),
// 					'status_remarks'=>$posted['remarks']
// 				];
// 				$statusData=[
// 					'order_product_id'=>$posted['id'],
// 					'status'=>$posted['status'],
// 					'status_date'=>date('Y-m-d H:i:s'),
// 					'remarks'=>$posted['remarks']
// 				];
// 			}
			
// 			if(OrderProduct::where('id',$posted['id'])->update($ostatusData)){
// 				$orderStatus = new OrderProductStatus;
				
// 				$orderStatus->fill($statusData);
// 				$orderStatus->save();
				
// 				return redirect()->back()->with("success","Order Status Successfully Updated.");
				
// 			} else {
// 				return redirect()->back()->with('error',"Sorry! Not Updated. Try again.");
// 			}
			
			
			
			
// 		} else {
// 			return redirect()->back()->with('error',"Invalid Credentials. Try again.");
// 		}
		
		
// 	}
	
	
	//print delivery Notes
	public function printDeliveryNote($id)
	{
	    if(empty($id) or !is_numeric($id)){
	        //checking if valid $id is provided
	        return redirect()->route('orders.index')->with("error","Invalid Credentials provided for Delivery Note.");
	    }
	    
	    $order = Order::with('order_products.order_status')->with('order_products.user')->find($id);
	    
	    if(empty($order)){
	        return redirect()->route('orders.index')->with("error","Invalid Order ID Provided.");
	    }
	    
	    if(User::isSeller()){
	        return view('seller/orders/printDeliveryNote',['order'=>$order]);
	    }
	    return view('orders/printDeliveryNote',['order'=>$order]);
	}
	
	//print invoice
	public function printInvoice($id)
	{
	    if(empty($id) or !is_numeric($id)){
	        //checking if valid $id is provided
	        return redirect()->route('orders.index')->with("error","Invalid Credentials provided for Delivery Note.");
	    }
	    
	    $order = Order::with('order_products.order_status')->with('order_products.user')->find($id);
	    
	    //echo"<pre>";print_r($order);exit;
	    
	    if(empty($order)){
	        return redirect()->route('orders.index')->with("error","Invalid Order ID Provided.");
	    }
	    
	    if(User::isSeller()){
	        return view('seller/orders/printInvoice',['order'=>$order]);
	    }
	    return view('orders/printInvoice',['order'=>$order]);
	    
	}
	
	public function markOrderStatus($id,$type)
	{
	    $order = Order::find($id);
	    
	    if(empty($order))
	    {
	        return redirect()->back()->with("error","Invalid Order ID Provided.");
	    }
	    
	    if($type)
	    {
	        $order->status = 'Acknowledged';
	        $title = "Order has been accepted by seller successfully.";
	    }
	    else 
	    {
	        $order->status = 'Rejected';
	        $title = "Order has been rejected by seller.";
	    }
	    
	    if($order->save())
	    {
	        $user = Auth::user();
	        $admin = User::where('role',"Master Admin")->first();
	        $seller = $order->order_products[0]->user;
	        
	        //send notification to browser
	        $user->sendNotification($admin->device_token,"New Order Placed by " . $user->name,"Order Placed");
	        $user->sendNotification($seller->device_token,"New Order Placed by " . $user->name,"Order Placed");
	        
	        // Sending push notification to user
	        if($user->notifications == 'Yes' && !empty($user->device_token))
	        {
	            $data = ['notification_type'=>'place order', 'id' => $order->id];
	            $user->sendNotification($user->device_token,$title,$data);
	        }
	    }
	    
	    return redirect()->back()->with("error","Order status changed successfully.");
	}
	
	
	/*
	 * Ajax Methods begins here
	*/
	public function getOrderStatusForm(Request $request){
		$posted = $request->all();
		
		if(empty($posted['opid'])){
			echo 0;
		} else {
			$orderProduct = Order::where('id',$posted['opid'])->with('order_status')->first();
			return view('orders/getOrderStatusForm',['product'=>$orderProduct]);
		}
	}

	public function getOrderStatusHistory(Request $request){
		$posted = $request->all();
		
		if(empty($posted['opid'])){
			echo 0;
		} else {
			$orderProduct = Order::where('id',$posted['opid'])->with('order_status')->first();
			return view('orders/getOrderStatusHistory',['product'=>$orderProduct]);
		}
	}

	/*Ajax Methods ends here*/
	
	
	/*
	 * APIs Begins here
	 */
	public function placeMyOrder(Request $request){
		//getting posted data
		
		$posted = $request->all();
		//print_r($posted);exit;
		$validator = Validator::make($request->all(), [
                    'address_id'=>'required|numeric',
		            'billing_address_id'=>'required|numeric',
		            'slot_time'=>'required',
		            'slot_date'=>'required',
                    'payment_type'=>'required',
                    'bag_amount'=>'required|numeric',
                    'tax'=>'required|numeric',
//                     'coupon_id'=>'required|numeric',
                    'service_charges'=>'required|numeric',
                    'total_amount'=>'required|numeric',
                    'products'=>['required','array',function ($attribute, $value, $fail) {
						           
								   $emsg = "";
								   //print_r($value);exit;
								   if(is_array($value)){
    								   	foreach($value as $pdata){
    										if(empty($pdata['variation_id'])){
    											$emsg.="variation_id is required, ";
    										} else {
    											if(!is_numeric($pdata['variation_id'])){
    												$emsg.="variation_id must be an integer., ";
    											}
    										}
    										if(empty($pdata['quantity'])){
    											$emsg.="quantity is required., ";
    										} else {
    											if(!is_numeric($pdata['quantity'])){
    												$emsg.="quantity must be a numeric value., ";
    											}
    										}
    									}
								   }
								   
									if($emsg!=""){
										$fail(substr($emsg,0,-2));
									}
									
						        }]
        ]);
		
		
		if ($validator->fails()) {
            $errors = $validator->errors()->getMessages();
			$error_message = "";
			foreach($errors as $fld=>$error){
				 $error_message = $error[0].", ";
			}
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
       
	   //getting delivery address
	   $address = Address::where(['id'=>$posted['address_id'],'user_id'=>Auth::user()->id])->first();
	   if(empty($address)){
	   	return response()->json(['status' => 0, 'message' => "Delivery Address does not belongs to current user."]);
	   }
	   
	   //getting billing address
	   $billingAddress = Address::where(['id'=>$posted['billing_address_id'],'user_id'=>Auth::user()->id])->first();
	   if(empty($billingAddress)){
	       return response()->json(['status' => 0, 'message' => "Billing Address does not belongs to current user."]);
	   }
	   
	   //getting slot details
	   if(isset($posted['slot_id'])){
    	   $slotDetail = TimeSlot::find($posted['slot_id']);
    	   
    	   if(empty($slotDetail)){
    	       return response()->json(['status' => 0, 'message' => "Please select a delivery slot."]);
    	   }
    	   $slotPrice = $slotDetail->price;
	   }else{
	       if(empty($posted['slot_price'])){
	           return response()->json(['status' => 0, 'message' => "Slot price is required."]);
	       }
	       $slotPrice = $posted['slot_price'];
	   }
	   
	   $order = [
	   	'order_number'=>$this->genOrderNumber(),
	   	'user_id'=>Auth::user()->id,
        'address'=>json_encode($address),
        'notes' => $posted['notes'] ?? '',
        'billing_address'=>json_encode($billingAddress),
	   	'payment_type'=>$posted['payment_type'],
	   	'payment_status'=>($posted['payment_type']=='COD')?'Pending':"Pending",
	   ];
	   
	   
	   $grandTotal = 0;
	   $bagAmount = 0;
	   $totalTax = 0;
	   
	   //for cart ids to be removed from cart
	   $cartIds = [];
	   
	   //for out of stock checking
	   $outOfStock=[];
	   
	   
	   //beginning the database transactions
	   DB::beginTransaction();
	   
	   //processing order products 
	   $orderProducts = [];
	   foreach($posted['products'] as $oProduct){
	   		
			//getting ordered quantity of product
			$pQuantity = $oProduct['quantity'];
			
			//getting product info
			$product = Sku::where('id',$oProduct['variation_id'])->with(['product.categories','product.tax','images','productSpecifications','currentBatch'])->first();
			
			if($pQuantity<=0){
				//quantity must be minimum positive 1 number
				return response()->json(['status' => 0, 'message' => "Invalid Quantity of product.",'data'=>$product]);
			}
// 			echo "<pre>";print_r($product->currentBatch->seller_id);exit;
			
			if(empty($product->currentBatch)){
				//if stock not available	
				$outOfStock[]=$product;
			} elseif($product->currentBatch->balance_quantity<$pQuantity) {
				//if ordered stock not available
				$outOfStock[]=$product;
			} 
			
			//Check store is open or not
			$sellerId = $product->currentBatch->seller_id;
			$seller = User::find($sellerId);
			if($seller->is_open == "No")
			{
			    return response()->json(['status' => 0, 'message' => "Sorry!, Store is closed."]);
			}
// 			$open = Address::isCurrentyOpen($seller->addresses[0]->open_time, $seller->addresses[0]->close_time);
// 			if(!$open || $seller->is_open == "No"){
// 			    return response()->json(['status' => 0, 'message' => "Sorry!, Store is closed."]);
// 			}
			
			if(!empty($outOfStock)){
				//omit further calculations if any product is out of stock
				continue;
			} else {
				//performing the calculations for product
				$unitPrice = $product->selling_price;
				$quantityPrice = round($unitPrice*$pQuantity,2);
				
				//coupon code calculations will goes here
				 
				//effective price
				$price = $quantityPrice; 
				$tax = 0;
// 				if($product->product->taxable == 'Yes'){
// 					//if Product is taxable
// 					//get tax rate and calculate tax
// 					$taxRate = $product->product->tax->rate;
// 					$tax = round(($price*$taxRate)/100,2);
// 				}
				
				//total amount for product
				$totalAmount = $price+$tax;
				$status = Order::STATE_PENDING;
								
				//updating order total variables
			    $grandTotal += $totalAmount;
				$bagAmount += $quantityPrice;
			   	$totalTax += $tax;
				
				
				$orderProducts[]=[
					'order_number'=>$order['order_number'],
					'sku_id'=>$oProduct['variation_id'],
				    'seller_id' =>$product->currentBatch->seller_id,
					'product'=>json_encode($product),
					'quantity'=>$pQuantity,
					'unit_price'=>$unitPrice,
					'price'=>$quantityPrice,
					'tax_info'=>($product->taxable=='Yes')?json_encode($product->tax):"",
					'tax'=>$tax,
					'coupon_code'=>null,
					'coupon_discount'=>0,
					'total_amount'=>$totalAmount,
					'status'=>$status,
					'status_date'=>date('Y-m-d H:i:s'),
					'created_at'=>date('Y-m-d H:i:s'),
					'updated_at'=>date('Y-m-d H:i:s')
				];
				
				//decreasing quantity from stock batch;
				Batch::find($product->currentBatch->id)->decrement('balance_quantity',$pQuantity);
				
				if(!empty($product->cart)){
					$cartIds[]=$product->cart->id;
				}
				
			}
			
	   }
	   
	   if(!empty($outOfStock)){
		   //stop placing order if any product is out of stock
		   //and returning the out of stock product
		   return response()->json(['status' => 0, 'message' => "Some Product(s) are out of stock OR ordered quantity not available.",'data'=>null]);
	   }
	   
	   $postAmount = $posted['total_amount'];
	   $bagAmountAfterDiscount = 0;
	   
	   //Calculate coupon discount if any
	   $discountPrice = 0;
	   if(!empty($posted['coupon_id'])){
	       
	       $offer = Offer::where('id',$posted['coupon_id'])->where('enabled','Yes')->where('end_date', '>=', date('Y-m-d'))->first();
	       if(empty($offer) || $bagAmount < $offer->min_amount){
	           return response()->json(['status' => 0, 'message' => "Offer not vaild."]);
	       }
	       if($offer->discount_type == 'flat'){
	           $discountPrice = $offer->amount;
	           $bagAmountAfterDiscount = $bagAmount - $discountPrice;
	           
	       }else{
	           
	           $discountPrice = round(($grandTotal * $offer->amount)/100,2);
	           if($discountPrice > $offer->discount_upto){
	               $discountPrice = $offer->discount_upto;
	           }
	           $bagAmountAfterDiscount = round($bagAmount - $discountPrice,2);
	       }
	       
	       //calculate grand total after apply discount on bag-amount if
	       $grandTotal =  $bagAmountAfterDiscount + $slotPrice + $posted['service_charges'];
	       $grandTotal = number_format($grandTotal, 2, '.', '');
	   }else{
	       
	       //calculate grand total without discount
	       $grandTotal =  $bagAmount + $slotPrice + $posted['service_charges'];
	       $grandTotal = number_format($grandTotal, 2, '.', '');
	   }
	   
	   
// 	   return response()->json(['b' => $bagAmount,'g'=>$grandTotal]);
	   
	   if(intval($postAmount) != intval($grandTotal))
	   {
	       //checking if amount calculations are same as shown on the application checkout screen
	       //and if the amount in the request is not tempeted
	       return response()->json(['status' => 0, 'message' => "Sorry! Order request seems to be tempered. Please check and try again.backend amount = " . $grandTotal . " and post amount = " . $postAmount]);
	   }
	   if(substr($postAmount, strpos($postAmount, ".") + 1) != substr($grandTotal, strpos($grandTotal, ".") + 1))
	   {
	       //checking if amount calculations are same as shown on the application checkout screen
	       //and if the amount in the request is not tempeted
	       return response()->json(['status' => 0, 'message' => "Sorry! Order request seems to be tempered. Please check and try again.backend amount = " . $grandTotal . " and post amount = " . $postAmount]);
	   }
	   
	   //updating order data with calculated numbers
	   $order['bag_amount'] = $bagAmount;
	   $order['tax']=$totalTax;
	   $order['coupon_discount']=$discountPrice;
	   $order['offer_json']= (!empty($posted['coupon_id'])) ? json_encode($offer) : "";
	   $order['total_amount']=$grandTotal;
	   $order['status']=Order::STATE_PENDING;
	   $order['slot_price']=$slotPrice;
	   $order['slot_time']=$posted['slot_time'] ?? "";
	   $order['delivery_date']=$posted['slot_date'] ?? "";
	   $order['is_contact_delivery'] = $posted['is_contact_delivery'] ?? "No";
	   $order['is_billing_contact_delivery'] = $posted['is_billing_contact_delivery'] ?? "No";
	   $order['slot_id']=$posted['next_day_slot_id'] ?? "";
	   $order['service_charges']=$posted['service_charges'];
	   
	   //return response()->json(['order'=>$order,'odata'=>$orderProducts]);
	   
	   
	   //finally saving order and order data
	   try {
	   		$currentOrder = new Order;
	   		$currentOrder->fill($order);
			$currentOrder->save();
			
			foreach($orderProducts as $key=>$oProduct){
				$orderProducts[$key]['order_id'] = $currentOrder->id;
			}
			
			OrderProduct::insert($orderProducts);
			
			if(!empty($cartIds)){
				//deleting products from cart
				Cart::destroy($cartIds);
			}
			
			DB::commit();
			
			return response()->json(['status' => 1, 'message' => "Order Placed Successfully.",'data'=>$currentOrder]);
	   } catch (\Exception $e) {
			DB::rollback();
			//dd($e);
			return response()->json(['status' => 0, 'message' => "Something Went Wrong. Try again."]);
	   }
	    
		
	}
	
	//get my Orders
	public function myOrders(){
	    $orders = Order::where('user_id',Auth::user()->id)->with(['order_status','order_products'])->orderBy('id','desc')->paginate(10); 
		if(empty($orders)){
			return response()->json(['status' => 1, 'message' => "No Order Found"]);
		} 
		
		return response()->json(['status' => 1, 'message' => "My Orders",'data'=>$orders]);
	}
	
	
	
	//get Order Details
	public function myOrderDetails($id){
		$order = Order::with('order_products.order_status')->where(['id' => $id, 'user_id' => Auth::user()->id])->first();
		if(empty($order)){
			return response()->json(['status' => 0, 'message' => "Invalid Order ID"]);
		} 
		
		return response()->json(['status' => 1, 'message' => "Order Details",'data'=>$order]);
	}
	
	
	
	//generating Order Number Whole Application wise unique
	public function genOrderNumber(){
		$ono = "OJ".rand(100000000,999999999);
		
		//validating for uniqueness
		$isExist = Order::where(['order_number'=>$ono])->count();
		if($isExist!=0){
			$ono = $this->genOrderNumber();
		} 
		return $ono;
	}
	
	// get order status details
	public function getOrderStatusDetail($id){
	    $order = Order::with('order_products.order_status')->where(['id' => $id, 'user_id' => Auth::user()->id])->first();
	    if(empty($order)){
	        return response()->json(['status' => 0, 'message' => "Invalid Order ID"]);
	    }
	    
	    return response()->json(['status' => 1, 'message' => "Order Status Details",'data'=>$order]);
	}
	
	// cancel order
	public function cancelOrder(Request $request)
	{
	    $validator = Validator::make($request->all(), [
	        'id'=>'required|numeric',
	        'cancel_reason'=>'required'
        ]);
	    
	    
	    if ($validator->fails()) {
	        $errors = $validator->errors()->getMessages();
	        $error_message = "";
	        foreach($errors as $error){
	            $error_message = $error[0].", ";
	        }
	        return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
	    }
	    
	    $order = Order::where(['id' => $request->id, 'user_id' => Auth::user()->id])->first();
	    if(empty($order) || $order->status == Order::STATE_CANCELED){
	        return response()->json(['status' => 0, 'message' => "This Order does not belongs to current user."]);
	    }
	    
	    $order->status = Order::STATE_CANCELED;
	    if($order->save()){
	        $orderStatus = new OrderStatus();
	        $orderStatus->order_id = $request->id;
	        $orderStatus->status = Order::STATE_CANCELED;
	        $orderStatus->status_date = date('Y-m-d h:i:s');
	        $orderStatus->remarks = $request->cancel_resign;
	        $orderStatus->save();
	        return response()->json(['status' => 1, 'message' => "Order Cancel successfully."]);
	    }else{
	        return response()->json(['status' => 1, 'message' => "Order not canceled."]);
	    }
	}
	
	
	/*
	 * Payment of orders
	 * params : stripe_token, order_id.
	 */
	public function pay(Request $request)
	{
	    if(empty($request->order_id)){
	        return response()->json(['status' => 0, 'message' => 'Order id is required']);
	    }
	    
	    $order = Order::find($request->order_id);
	   
	    if(empty($order) || $order->payment_status != "Pending"){
	        return response()->json(['status' => 0, 'message' => 'Order not found']);
	    }	
	    
	    $amount = $order->total_amount * 100;
	    $orderNumber = $order->order_number;
	    $user = User::find($order->user_id);
	    
	    if(empty($user)){
	        return response()->json(['status' => 0, 'message' => 'User not found']);
	    }
	    
	    Stripe::setApiKey(config('services.stripe.secret'));
	    try {
	        
	        if(!empty($request->stripe_token)){
	            $token = $request->stripe_token;
	            
                // create customer on stripe if not created
    	        if(empty($user->stripe_customer_id)){
                    // Create a New Customer:
        	        $customer = \Stripe\Customer::create([
//         	            'source' => $token,
        	            'email' => $user->email,
        	        ]);
        	        
        	        // Save the customer ID in a database for later.
        	        $user->stripe_customer_id = $customer->id;
        	        $user->save();
    	        }
    	        
    	        //saving new card and make it default
    	        if(!empty($user->stripe_customer_id) && !empty($request->new_card)){
    	            
    	            $card = \Stripe\Customer::createSource(
    	                $user->stripe_customer_id,
        	            [
        	                'source' => $token,
        	            ]
    	            );
    	            
    	            \Stripe\Customer::update(
    	                $user->stripe_customer_id,
    	                [
    	                    'default_source' => $card->id,
    	                ]
                    );
    	        }
	        }
	        
	        //make already saved card default
	        if(!empty($user->stripe_customer_id) && !empty($request->card_id)){
    	        \Stripe\Customer::update(
    	            $user->stripe_customer_id,
    	            [
    	                'default_source' => $request->card_id,
    	            ]
                );
	        }
	        
	        $charge = \Stripe\Charge::create([
	            'customer' => $user->stripe_customer_id,
	            'amount' => $amount,
	            'currency' => 'GBP',
	            'description' => 'Order Number : ' . $orderNumber,
// 	            'source' => $token,
	        ]);

	        return response()->json(['status' => 0, 'data' => $charge]);
	        
	        if($charge['status'] == 'succeeded') {
	            $order->payment_status = "Done";
	            $order->payment_json = json_encode($charge);
	            $order->save();
	            
	            
	            $user = Auth::user();
	            //send notification to browser
	            $admin = User::where('role',"Master Admin")->first();
	            $seller = $order->order_products[0]->user;
	           
	            $user->sendNotification($admin->device_token,"New Order Placed by " . $user->name,"Order Placed");
	            $user->sendNotification($seller->device_token,"New Order Placed by " . $user->name,"Order Placed");
	            
	            // Sending push notification
	            if($user->notifications == 'Yes' && !empty($user->device_token)){
	                $title = "Order has been placed successfully.";
	                $data = ['notification_type'=>'place order', 'id' => $order->id];
	                $user->sendNotification($user->device_token,$title,$data);
	            }
	            
	            //Sending order accept mail to user
	            $userEmail = $user->email;
	            if(!empty($userEmail))
	            {
	                $email_content['order'] = $order;
	                $email_content['name'] = $order->user->name;
	                $html = view('mail.userOrderDetails', compact('email_content'))->render();
	                $emailArr['to_email'] = $userEmail;
	                $emailArr['to_user_name'] =  $user->name;
	                $emailArr['subject'] = "Your order is confirmed 🎈";
	                User::sendEmail($emailArr, $html);
	            }
	            
	            //Sending order accept mail to admin
	            $email_content['order'] = $order;
	            $email_content['name'] = $order->user->name;
	            $html = view('mail.adminOrderDetails', compact('email_content'))->render();
	            $emailArr['to_email'] = 'mariam@my-oja.com';
	            $emailArr['to_user_name'] = 'mariam';
	            $emailArr['subject'] = "New order " . $order->order_number ." placed successfully.";
	            User::sendEmail($emailArr, $html);
	            
	            
	            return response()->json(['status' => 1, 'message' => "Payment done.",'data' => $charge->receipt_url]);
	        } else {
	            $msg = 'Money not add in wallet!!';
	            $status = 0;
	        }
	    } catch (\Exception $e) {
	        $status = 2;
	        $msg =  $e->getMessage();
	    } catch(\Cartalyst\Stripe\Exception\CardErrorException $e) {
	        $status = 3;
	        $msg =  $e->getMessage();
	    } catch(\Cartalyst\Stripe\Exception\MissingParameterException $e) {
	        $status = 4;
	        $msg = $e->getMessage();
	    }
	    
	    return response()->json(['status' => $status, 'message' => $msg]);
	}
	
	public function deleteCard(Request $request)
	{
	    if(empty( $request->card_id)){
	        return response()->json(['status' => 0, 'message' => 'Card id is required.']);
	    }
	    
	    $user = User::find(Auth::user()->id);
	    
	    if(empty($user) || empty($user->stripe_customer_id)){
	        return response()->json(['status' => 0, 'message' => 'card not found.']);
	    }
	    
	    $stripe = new \Stripe\StripeClient(
	        config('services.stripe.secret')
        );
	    
	    $stripe->customers->deleteSource(
	        $user->stripe_customer_id,
	        $request->card_id
        );
	    
	    return response()->json(['status' => 1, 'message' => 'card deleted']);
	}
	
	public function getSavedCards()
	{
	    $user = User::find(Auth::user()->id);
	    
	    if(empty($user) || empty($user->stripe_customer_id)){
	        return response()->json(['status' => 0, 'message' => 'No card found']);
	    }
	    
	    $stripe = new \Stripe\StripeClient(
	        config('services.stripe.secret')
        );
	    
	    $cards = $stripe->customers->allSources(
	        $user->stripe_customer_id,
	        ['object' => 'card', 'limit' => 30]
        );
	    
	    return response()->json(['status' => 1, 'cards' => $cards]);
	}
	
	public function orderStatuses(Request $request)
	{
	    if(empty($request->order_id)){
	        return response()->json(['status' => 0, 'message' => 'Order id is required.']);
	    }
	    
	    $order = Order::find($request->order_id);
	    if(empty($order)){
	        return response()->json(['status' => 0, 'message' => 'Order not found.']);
	    }
	    $storeName = ($order->order_products[0]->user) ? $order->order_products[0]->user->name : "Store";
	    
	    $data['order_number'] = $order->order_number;
	    $data['store'] = $storeName;
	    $data['store_lat'] = ($order->order_products[0]->user && $order->order_products[0]->user->addresses) ? $order->order_products[0]->user->addresses[0]->latitude : "";
	    $data['store_long'] = ($order->order_products[0]->user && $order->order_products[0]->user->addresses) ? $order->order_products[0]->user->addresses[0]->longitude : "";;
	    $data['estimated_time'] = $order->slot_time ?? "";
	    
	    $userAddress = json_decode($order->address);
	    $data['user_lat'] = $userAddress->latitude ?? "";
	    $data['user_long'] = $userAddress->longitude ?? "";
	    $orderStatuses = OrderStatus::where('order_id',$request->order_id)->get();
	    
	    $data['orderStatus'] = [];
	    foreach ($orderStatuses as $key => $orderStatus)
	    {
	        $msg = "";
	        
	        if($orderStatus->status == Order::STATE_CONFIRMED)
	        {
	            $msg = "Your order is confirmed by " . $storeName;
	        }
	        else if ($orderStatus->status == Order::STATE_PROCESSING)
	        {
	            $msg = "Your order is being prepared by " . $storeName;
	        }
	        else if ($orderStatus->status == Order::STATE_OUT_FOR_DELIVERY)
	        {
	            $msg = "Your order is out for delivery.";
	        }
	        else if ($orderStatus->status == Order::STATE_DELIVERED)
	        {
	            $msg = "Your order has been delivered successfully";
	        }
	        else if ($orderStatus->status == Order::STATE_REJECTED)
	        {
	            $msg =  $storeName . "Cannot accept your order right now.";
	        }
	        else
	        {
	            $msg = $orderStatus->status;
	        }
	        
	        $data['orderStatus'][$key]['status'] = $msg;
	        $data['orderStatus'][$key]['remarks'] = $orderStatus->remarks;
	        $data['orderStatus'][$key]['status_date'] = $orderStatus->status_date;
	    }
	    
	    return response()->json(['status' => 1, 'data' => $data]);
	}
	
	
	/*
	 * APIs Ends here
	 */
}
