<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Product;
use Validator;

class ProductApiController extends Controller
{


    /**
     * Store a newly created product in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */    

    public function storeProduct(Request $request){
        
        $validator = Validator::make($request->all(),[                
                'skuId'           =>      'required',
                'sellerId'       =>      'required',
                'quantity'        =>      'required',
                'sellerPrice'        =>      'required',
                'marginPercentage'        =>      'required',
                'disabled'        =>      'required',
                'userId'        =>      'required',
            ]);

        if($validator->fails()) {
            return response()->json(["status" => "failed", "message" => "Please enter all required fields."]);
        }

        if($request->disabled == 'false'){
            $enabled = 'true';
        }else{
            $enabled = 'false';
        }
        $productDataArray = array(
            'sku'  => $request->skuId,
            'merchant_id'    => $request->sellerId,
            'stock_quantity' => $request->quantity,
            'sale_price' => $request->sellerPrice,
            //'marginPercentage' => $request->marginPercentage,
            'enabled' => $enabled,
            'created_by' => $request->userId,
        );

        $product = Product::create($productDataArray);

        if(!is_null($product)){
            return response()->json(["status"=>200,"success"=>true,"message"=>"Product Added successfully","stockItems"=>[$product]]);
        }
        else{
            return response()->json(["status"=>"failed","success"=>false,"message"=>"Failed to Add Product"]);
        }
    }

    public function updateProduct(Request $request){
        
        $validator = Validator::make($request->all(),[                
                'productId' => 'required'                
        ]);

        if($validator->fails()) {
            return response()->json(["status" => "failed", "message" => "Please enter required fields."]);
        }

        if($request->disabled == 'false'){
            $enabled = 'true';
        }else{
            $enabled = 'false';
        }
        $productDataArray = array(
            'id'  => $request->productId,
            'sku'  => $request->skuId,
            'merchant_id'    => $request->sellerId,
            'stock_quantity' => $request->quantity,
            'sale_price' => $request->sellerPrice,
            //'marginPercentage' => $request->marginPercentage,
            'enabled' => $enabled,
            'created_by' => $request->userId,
        );

        $product = Product::where("id","=",$request->productId)->update($productDataArray);
        
        return response()->json(["status"=>200,"success"=>true,"message"=>"Product updated successfully"]);
       
    }

}
