<?php

namespace App\Http\Controllers;
use App\TimeSlot;
use Illuminate\Http\Request;
use Validator;
use DB;
use App\User;
use App\Cart;
use Illuminate\Support\Facades\Auth;
use App\Address;
use App\TimeSlotPrice;
use App\Settings;
use App\Order;


class TimeSlotController extends Controller
{
    
    public function __construct()
    {
        $this->middleware('auth');
    }
    
    
    public function timeSlotsList(Request $request)
    {
        
        if (count($request->all()) > 1) {
            $data['timeSlots'] = TimeSlot::where($request->ftype, 'like', '%' . $request->stext . '%')->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['timeSlots'] = TimeSlot::where('is_today_slot',null)->paginate(400);
            $data['ftype'] = "";
            $data['stext'] = "";
        }
        
        return view('time_slot/timeSlotList', $data);
    }
    
    public function todayTimeSlotsList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['timeSlots'] = TimeSlot::where($request->ftype, 'like', '%' . $request->stext . '%')->where('is_today_slot','yes')->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['timeSlots'] = TimeSlot::where('is_today_slot','yes')->paginate(10);
            $data['ftype'] = "";
            $data['stext'] = "";
        }
        
        return view('time_slot/todayTimeSlotsList', $data);
    }
    
    public function addTimeSlot()
    {
        return view('time_slot/addTimeSlot');
    }
    
    public function addTodayTimeSlot()
    {
        return view('time_slot/addTodayTimeSlot');
    }
    
    public function saveTimeSlot(Request $request)
    {
        $posted = collect($request->all())->toArray();
        
//             $request->validate([
//                 //'price' => 'required',
//                 'start_time' => 'required',
//                 'end_time' => 'required'
//             ]);
        
        $price = $posted['price'];
        
        $timeSlot = new TimeSlot();
        
        if(isset($posted['is_any_time']))
        {
            $timeSlot->is_any_time = 'Yes';
        }else{
            $timeSlot->start_time = $posted['start_time'];
            $timeSlot->end_time =$posted['end_time'];
        }
        $timeSlot->enabled = $posted['enabled'];
        
        if ($timeSlot->save()) {
            
            foreach ($posted['day'] as $key => $day)
            {
                $slotPrice = new TimeSlotPrice();
                $slotPrice->day = $key;
                $slotPrice->price = $day ?? $price;
                $slotPrice->time_slot_id = $timeSlot->id;
                $slotPrice->save();
            }
            
            return redirect()->route('timeSlotsList')->with('success', 'Time Slot created successfully.');
        } else {
            return back()->with('error', 'Time Slot has not been created.');
        }
    }
    
    public function editTimeSlot($id)
    {
        $timeSlot = TimeSlot::where([
            'id' => $id
        ])->with('timeSlotPrice')->first();
        
        if (! $timeSlot) {
            return redirect()->route('timeSlotsList')->with('warning', 'Invalid Time Slot to edit');
        }
        
        if($timeSlot->is_today_slot == "Yes"){
            return view('time_slot/editTodayTimeSlot', [
                'timeSlot' => $timeSlot
            ]);
        }
        
        return view('time_slot/editTimeSlot', [
            'timeSlot' => $timeSlot
        ]);
    }
    
    public function saveEditTimeSlot(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();
        
//         $request->validate([
// //             'price' => 'required',
//             'start_time' => 'required',
//             'end_time' => 'required'
//         ]);
        
        $timeSlot = TimeSlot::where([
            'id' => $id
        ])->first();
        if (! $timeSlot) {
            return redirect()->route('timeSlotsList')->with('warning', 'Invalid Time Slot to edit');
        }
        $price = $posted['price'];
        
        if(isset($posted['is_any_time']))
        {
            $timeSlot->is_any_time = 'Yes';
        }else{
            $timeSlot->start_time = $posted['start_time'];
            $timeSlot->end_time =$posted['end_time'];
            $timeSlot->is_any_time = '';
        }
        
        $timeSlot->enabled = $posted['enabled'];
        
        if ($timeSlot->save()) {
            
           
            foreach ($posted['day'] as $key => $day)
            {
                $slotPrice = TimeSlotPrice::find($key);
                $slotPrice->price = $day ?? $price;
                if (isset($posted['is_full']) && array_key_exists($key,$posted['is_full']))
                {
                    $slotPrice->is_full = true;
                }else {
                    $slotPrice->is_full = false;
                }
                $slotPrice->save();
            }
            
            return redirect()->route('timeSlotsList')->with('success', 'Time Slot updated successfully.');
        } else {
            return back()->with('error', 'Time Slot has not been modified.');
        }
    }
    
    public function saveEditTodayTimeSlot(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();
        
        $setting = Settings::first();
        $miles = $setting->max_delivery_miles ?? 0;
        
        $request->validate([
            'price' => 'required',
            'title' => 'required',
            'delivery_mile' => 'required|lte:'.$miles,
        ]);
        
        $timeSlot = TimeSlot::where([
            'id' => $id
        ])->first();
        
        if (! $timeSlot) {
            return redirect()->route('timeSlotsList')->with('warning', 'Invalid Time Slot to edit');
        }
        
        $timeSlot->price = $posted['price'];
        $timeSlot->delivery_mile = $posted['delivery_mile'];
        $timeSlot->enabled = $posted['enabled'];
        
        if($timeSlot->hour == TimeSlot::EVENING_SLOT){
            $timeSlot->start_time = $posted['start_time'];
            $timeSlot->end_time = $posted['end_time'];
        }
        
        if(isset($posted['title'])){
            $timeSlot->title = $posted['title'];
        }
        
        if ($timeSlot->save()) {
        
            return redirect()->route('todayTimeSlotsList')->with('success', 'Today Time Slot updated successfully.');
        } else {
            return back()->with('error', 'Time Slot has not been modified.');
        }
    }
    
    public function deleteTimeSlot(Request $request)
    {
        if (TimeSlot::find($request->id)->forceDelete()) {
            
            return redirect()->back()->with('success', 'Time Slot has been Deleted!');
        } else {
            return redirect()->back()->with('error', 'Time Slot has not been Deleted!');
        }
    }
    
    //used to update enabled field of today time slot from list page
    public function markSlotEnabled(Request $request,$id,$enb)
    {
        TimeSlot::where('id',$id)->update(['enabled'=>$enb]);
        
        return redirect()->back()->with('success','Updated');
    }
    
    
    
    /**
     * API Start Here
     */
    public function getTimeSlots()
    {
        $setting = Settings::first();
        $timeSlots = TimeSlot::where(['is_today_slot' => null, 'enabled' => "Yes"])->with('timeSlotPrice')->paginate(50);
        if(empty($timeSlots)){
            return response()->json(['status' => 0, 'message' => "No record found"]);
        }
        
        $finalSlot = [];
        foreach ($timeSlots as $timeSlot)
        {
            $slots = [];
            foreach ($timeSlot->timeSlotPrice as $key => $slotPrice)
            {
                $slotOrder = Order::where([['status','!=','Shipped'],['status','!=','Rejected'],['status','!=','Delivered']])->where('slot_id',$slotPrice->id)->count();
                
                $slots[$key]['id'] = $slotPrice->id;
                $slots[$key]['Time'] = $slotPrice->day;
                $slots[$key]['Price'] = $slotPrice->price;
                
                if($slotPrice->is_full || $slotOrder >= $setting['slot_limit'])
                {
                    $slots[$key]['is_full'] = true;
                }else{
                    $slots[$key]['is_full'] = false;
                }
                
//                 $slots[$key]['is_full'] = ($slotOrder >= $setting['slot_limit']) ? true : false;
            }
            if(!empty($timeSlot->is_any_time)){
                $finalSlot['Any Time'] = $slots;
            }else{
                $finalSlot[$timeSlot->start_time . " - " . $timeSlot->end_time] = $slots;
            }
        }
        
        $deliveryMiles = $setting->next_day_delivery_miles;
        
        return response()->json(['status' => 1, 'message' => "Time slot list",'data'=>$finalSlot,'deliveryMiles'=>$deliveryMiles]);
    }
    
    public function getTodayTimeSlots()
    {
        $cart = Cart::where(['user_id'=>Auth::user()->id])->first();
        
        if(empty($cart)){
            return response()->json(['status' => 0, 'message' => 'No item found in cart.']);
        }
        
        if(empty($cart->variation) || empty($cart->variation->currentBatch)){
            return response()->json(['status' => 0, 'message' => 'Request seems to be tempered.']);
        }
        
        $sellerId = $cart->variation->currentBatch->seller_id;
        
        if(empty($sellerId)){
            return response()->json(['status' => 0, 'message' => "Seller not found."]);
        }
        
        $user = User::where('id',$sellerId)->first();
        $openTime = $user->addresses[0]->open_time;
        $closeTime = $user->addresses[0]->close_time;
        
//         $open = Address::isCurrentyOpen($openTime, $closeTime);
        if($user->is_open == "No"){
            return response()->json(['status' => 0, 'message' => "Store Closed.",'is_closed' => "Yes"]);
        }
        
        $time = new TimeSlot();
        $data['slots'] = $time->getSlotFromCurrentTime($closeTime);
        
        //Setting keys for App side
        $anyTime = TimeSlot::where('is_today_slot','yes')->whereNull('hour')->where('enabled','yes')->first();
        if(!empty($anyTime))
        {
            $data['anytime_slot']['id'] = $anyTime->id;
            $data['anytime_slot']['name'] = $anyTime->title;
            $data['anytime_slot']['delivery_mile'] = $anyTime->delivery_mile;
            $data['anytime_slot']['enabled'] = $anyTime->enabled;
            $data['anytime_slot']['price'] = $anyTime->price;
            $data['anytime_slot']['hour'] = $anyTime->hour;
        }
        
        $setting = Settings::first();
        $data['order_cutoff'] = ($setting->order_cut_off_time) ?? "";
        
        return response()->json(['status' => 1, 'message' => "Today Time slot list",'data'=>$data]);
    }
}
