<?php
namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends Model
{

    use SoftDeletes;

    protected $guarded = [];

    protected $hidden = [
        'deleted_at',
        'category_id'
    ];

    public function getImageAttribute()
    {
        return $this->skus()
            ->first()
            ->images()
            ->first()->name;
    }

    public function skus()
    {
        return $this->hasMany('App\Sku', 'product_id');
    }

    // duplicate of skus() relation for APIs
    public function variations()
    {
        return $this->hasMany('App\Sku', 'product_id');
    }

    public function tax()
    {
        return $this->belongsTo('App\Tax', 'tax_id')->where('enabled', 'Yes');
    }
    
    public function brand()
    {
        return $this->belongsTo('App\Brand', 'brand_id');
    }

    public function category()
    {
        return $this->belongsTo('App\Category', 'category_id')->with('category');
    }

    public function categories()
    {
        return $this->belongsToMany('App\Category')->with('category');
    }
    
    public function country(){
        return $this->belongsTo('App\Country','country_id');
    }
}
