<?php
namespace App;

use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Auth;
use Laravel\Passport\HasApiTokens;

class Seller extends Authenticatable
{

    use SoftDeletes;

    use HasApiTokens,Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    /*
     * protected $fillable = [
     * '*'
     * ];
     */
    protected $guarded = [];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token'
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime'
    ];

    public function addresses()
    {
        return $this->hasMany('App\Address', 'user_id');
    }

    public function orders()
    {
        return $this->hasMany('App\Order', 'user_id');
    }

    public function batches()
    {
        return $this->hasMany('App\Batch', 'seller_id');
    }

    public static function isSuperAdmin()
    {
        if (Auth::user()->role == "Master Admin") {
            return true;
        }
        return false;
    }

    public static function isSeller()
    {
        if (Auth::user()->role == "Seller") {
            return true;
        }
        return false;
    }

    public static function isOwnAccount($id)
    {
        if (Auth::user()->id == $id) {
            return true;
        }
        return false;
    }

    public function deliveryAddresses()
    {
        return $this->belongsToMany('App\DeliveryAddress', 'seller_delivery_addresses', 'seller_id', 'delivery_address_id');
    }

    public function sellerCultures()
    {
        return $this->belongsToMany('App\Culture', 'seller_cultures', 'seller_id', 'culture_id');
    }
}
