<?php
namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;

class Sku extends Model
{

    use SoftDeletes;

    protected $table = "product_skus";

    protected $guarded = [];

    protected $appends = [
        'current_batch_quantity',
        'selling_price',
        'price_with_tax',
        'balance_quantity'
    ];

    protected $hidden = [
        'deleted_at',
        'category_id'
    ];

    function getBalanceQuantityAttribute()
    {
        $bal = 0;
        if (! empty($this->batches())) {
            $bal = $this->batches()->sum('balance_quantity');
        }
        return $bal;
    }

    function getCurrentBatchQuantityAttribute()
    {
        $bal = 0;
        if (! empty($this->currentBatch)) {
            $bal = $this->currentBatch->balance_quantity;
        }
        return $bal;
    }

    function getSellingPriceAttribute()
    {
        $b = 0;
        if (! empty($this->currentBatch)) {
            $b = $this->currentBatch->base_rate;
        }
        return $b;
    }

    function getPriceWithTaxAttribute()
    {
        $total = 0;
        if (! empty($this->currentBatch)) {
            $total = $this->currentBatch->base_rate;
            if (! empty($this->product->tax)) {
                $total = $total + round(($total * $this->product->tax->rate) / 100, 2);
            }
        }

        return $total;
    }

    /* relations */
    public function product()
    {
        return $this->belongsTo('App\Product', 'product_id');
    }

    public function images()
    {
        return $this->hasMany('App\ProductImage', 'sku_id');
    }

    public function productSpecifications()
    {
        return $this->hasMany('App\ProductSpecification', 'sku_id')
            ->whereNull('parent_id')
            ->with('productSpecifications');
    }

    public function wishList()
    {
        return $this->hasOne('App\Wishlist', 'sku_id')->where('user_id', Auth::id());
    }

    public function cart()
    {
        return $this->hasOne('App\Cart', 'sku_id')->where('user_id', Auth::id());
    }

    public function banners()
    {
        return $this->belongsToMany('App\Banner', 'banner_sku');
    }

    public function sections()
    {
        return $this->belongsToMany('App\Section', 'section_sku');
    }

    /* Inventory associations */
    public function batches()
    {
        return $this->hasMany('App\Batch', 'sku_id')
            ->where('enabled', 'Yes')
            ->orderBy('sale_order');
    }

    public function currentBatch()
    {
        return $this->hasOne('App\Batch', 'sku_id')
            ->where('balance_quantity', '>', '0')
            ->where('enabled', 'Yes')
            ->orderBy('sale_order');
    }

    public static function productSkus()
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => 'http://18.222.180.36:8080/productSkus',
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => '',
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 0,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = curl_exec($curl);
        if (curl_errno($curl)) {
                    echo $error_msg = curl_error($curl);                 die;
                    //Log::error($error_msg);
                    //return redirect()->back()->with('error','Something went wrong!');                                   
                }
        curl_close($curl);       
        
        return $response;
    }
    /* Inventory associations ends */
    /* relation ends */
}
