<?php

namespace App;

use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Auth;
use Laravel\Passport\HasApiTokens;
use Twilio\Rest\Client;
use Twilio\Exceptions\TwilioException;


class User extends Authenticatable
{
	
	use SoftDeletes;
	
    use HasApiTokens,Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    /*
    protected $fillable = [
           '*'
        ];*/
        
    /* Use for differentiate User and Admin Login Functionality */    
    const  AdminLogin = 1;
    const  merchantLogin= 0;    
    protected $guarded = [];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];
	
    protected $appends = [
            'seller_rating'
    ];
	
	public function addresses(){
		return $this->hasMany('App\Address','user_id');
	}
	
	public function orders(){
		return $this->hasMany('App\Order','user_id');
	}
	
	public function batches(){
	    return $this->hasMany('App\Batch','seller_id');
	}
	
	public function ratings(){
	    return $this->hasMany('App\Rating','seller_id','id');
	}
	
	public static function isSuperAdmin()
	{
	    if(Auth::user()->role == "Master Admin"){
	        return true;
	    }
	    return false;
	}
	
	public static function isSeller()
	{
	    if(Auth::user()->role == "Seller"){
	        return true;
	    }
	    return false;
	}
	
	public static function isOwnAccount($id)
	{
	    if(Auth::user()->id == $id){
	        return true;
	    }
	    return false;
	}
	
	public function deliveryAddresses()
	{
	    return $this->belongsToMany('App\DeliveryAddress','seller_delivery_addresses', 'seller_id', 'delivery_address_id');
	}
	
	public function sellerCultures()
	{
	    return $this->belongsToMany('App\Culture','seller_cultures', 'seller_id', 'culture_id');
	}
	
	public static function getLoginUser()
	{
	    $user = User::find(Auth::user()->id);
	    
	    if(!empty($user)){
	        return $user;
	    }
	    return null;
	}
	
	// sending notification using one signal
	public static function sendNotification($playerId,$title,$data)
	{    
	    $content = array(
	        "en" => $title
	    );
	    
	    $fields = array(
	        'app_id' => "d0fe8d16-9657-421d-bfed-5a4c4b09be58",
	        'include_player_ids' => array($playerId),
	        'data' => array("body" => $data),
	        'contents' => $content
	    );
	    
	    $fields = json_encode($fields);
	    
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, "https://onesignal.com/api/v1/notifications");
	    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json; charset=utf-8'));
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
	    curl_setopt($ch, CURLOPT_HEADER, FALSE);
	    curl_setopt($ch, CURLOPT_POST, TRUE);
	    curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
	    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
	    
	    $response = curl_exec($ch);
	    curl_close($ch);
	    
	    return json_decode($response);
	}
	
	//Sending SMS to app using Twilio
	public static function sendSms($phoneNumber,$body)
	{
	    $response = [];
	    $accountSid = config('app.twilio')['TWILIO_ACCOUNT_SID'];
	    $authToken  = config('app.twilio')['TWILIO_AUTH_TOKEN'];
	    //         $appSid     = config('app.twilio')['TWILIO_APP_SID'];
	    $client = new Client($accountSid, $authToken);
	    try
	    {
	        // Use the client to do fun stuff like send text messages!
	        $client->messages->create(
	            // the number you'd like to send the message to
	            $phoneNumber,
	            array(
	                // A Twilio phone number you purchased at twilio.com/console
	                'from' => '+44 7476 560178',
	                // the body of the text message you'd like to send
	                'body' => $body
	            )
            );
	    }
	    catch (TwilioException  $e)
	    {
	        $response['status'] = 0;
	        
	        if($e->getCode() == '21211'){
	            $response['error']= "Imvaild phone number.";
	        }
	        $response['error'] = $e->getMessage();
	        
	        return $response;
	    }
	    
	    $response['status'] = 1;
	    return $response;
	}
	
	function getsellerRatingAttribute() {
	    
	    return Rating::where('seller_id',$this->id)->avg('rating');
	}
	
	//Return distance in miles
	function getStoreDistanceFromUser($userLat, $userLong)
	{
	    if(!isset($this->addresses[0])){
	        return false;
	    }
	   
	    $sellerLat = $this->addresses[0]->latitude; 
	    $sellerLong = $this->addresses[0]->longitude; 
	    
         // Calculate distance between latitude and longitude in miles
	    $latitude1    = $sellerLat;
	    $longitude1    = $sellerLong;
	    $latitude2        = $userLat;
	    $longitude2    = $userLong;
	    
	    $theta = $longitude1 - $longitude2;
	    $distance = (sin(deg2rad($latitude1)) * sin(deg2rad($latitude2))) + (cos(deg2rad($latitude1)) * cos(deg2rad($latitude2)) * cos(deg2rad($theta)));
	    $distance = acos($distance);
	    $distance = rad2deg($distance);
	    $distance = $distance * 60 * 1.1515; 
	  
	    return (round($distance,2)); 
	}
	
	//Sending Email using Sendgrid 
	public static function sendEmail($emailArr,$html)
	{
	    $email = new \SendGrid\Mail\Mail();
	    $email->setFrom("Orders@my-oja.com", "Oja");
	    
	    $email->setSubject($emailArr['subject']);
	    $email->addTo($emailArr['to_email'], $emailArr['to_user_name']); 
	    $email->addContent(
	        "text/html", $html
        );
	    $sendgrid = new \SendGrid('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');
	    
	    try {
	        $response = $sendgrid->send($email);
	        return $response->statusCode();
	        
	    } catch (Exception $e) {
	        return 'Caught exception: '. $e->getMessage() ."\n";
	    }
	}
}
