@extends('layouts.app')

@section('content')
<div class="content-page">
	<div class="content">

		<!-- Start Content-->
		<div class="container-fluid">
			<!-- start page title -->
			<div class="row mt-2">
				<div class="col-8 col-sm-8 col-md-8 col-lg-8">
					<div class="page-title-box">
						<h4>Stock Batches</h4>
						<ol class="breadcrumb m-0">
							<li class="breadcrumb-item">
								<a href="{{route('dashboard')}}">Dashboard</a>
							</li>
							<li class="breadcrumb-item active">
								Stock Batches
							</li>
						</ol>
					</div>
				</div>
			</div>
			<!-- end page title -->
			
			<div class="mb-1" style="overflow:visible;">
				<span class="float-right">
					<a href="{{route('batches.create')}}" class="btn btn-info btn-sm" role="button">
						<i class="fa fa-plus mr-1"></i>
						Add Product Stock
					</a>
				</span>
				<form class="form-inline" action="{{route('batches.index')}}">
					@csrf
					@method('get')
					<div class="input-group">
					  <div class="input-group-prepend">
					  	<select class="form-control" name="search_type" id="example-select">
							<option <?= (request()->search_type == "name") ? 'selected' : ''?> value="name">Product Name</option>
							<option <?= (request()->search_type == "sku") ? 'selected' : ''?> value="sku">Product SKU</option>
							<option <?= (request()->search_type == "batch") ? 'selected' : ''?> value="batch">Batch</option>
						</select>
					  </div>
					  <input type="text" class="form-control" name="stext" value="{{request()->stext}}" placeholder="Search Keyword" aria-label="Recipient's username" aria-describedby="button-addon2">
					  <div class="input-group-append">
					    <button class="btn btn-secondary" type="submit" id="button-addon2">Search</button>
						<a href="{{route('batches.index')}}" class="btn btn-info btn-rounded waves-effect waves-light float-right">
							Clear
						</a>
					  </div>
					</div>
				</form>
			</div>
			
			<div class="table-responsive">
				<table class="table mb-0 table-bordered " >
					<thead class="thead-dark">
						<tr>
							<th>Batch No</th>
							<th>Product</th>
							<th>Quantity</th>
							<th>Balance Quantity</th>
							<th>Product MRP</th>
<!-- 							<th>Margin Ratio (%)</th> -->
							<th>Base Rate</th>
							<th>Seller</th>
							<th>Enabled</th>
							<th class="text-right">Action</th>
						</tr>
					</thead>
					<tbody>
						<?php if(count($batches)==0): ?>
							<tr class="table-info">
								<td colspan='11' class="text-center">
									No Records Found
								</td>
							</tr>
						<?php endif; ?>
						<?php foreach($batches as $batch): ?>
							
							<tr>
								<td>
									{{$batch->batch_number}}
								</td>
								<td>
									@if(empty($batch->product))
										No Product Associated
									@else
										{{$batch->product->product->name}} <br>
										({{$batch->product->sku}})
									@endif
								
								</td>
								<td>{{$batch->quantity}}</td>
								<td>
									{{ $batch->balance_quantity }}
								</td>
								<td >{{$batch->product_mrp}}</td>
<!-- 								<td>{{$batch->margin_ratio}}</td> -->
								<td>{{$batch->base_rate}}</td>
								<td><?=  isset($batch['users']) ? $batch['users']->name : ""?></td>
								<td>
									{{$batch->enabled}}
									@if($batch->enabled=='Yes')
										<?php $pplr = 'No'; ?>
									@else 
										<?php $pplr = 'Yes'; ?>
									@endif
									<form style="display:block;" action="{{route('markBatchEnabled',[$batch->id,$pplr])}}" method="get">
										@csrf
										@method('GET')
										<button type="submit" class="btn btn-xs btn-warning" data-toggle="tooltip" title="Change to {{$pplr}}">
											<i class="fa fa-recycle"></i>
										</button>
									</form>
								</td>
								<td class="text-right">
									<a href="{{route('batches.show',$batch->id)}}">
										<button type="button" class="btn btn-secondary waves-effect waves-light">
											<i class="mdi mdi-eye"></i>
										</button>
									</a>
									<button class="btn btn-blue waves-effect waves-light updateBatchBtn" data-url="{{route('getEditForm',$batch->id)}}" type="button">
                                		<i class="mdi mdi-pencil"></i>
                                	</button>
									<form style="display:inline-block;" onsubmit="return confirm('Are you sure to delete <?php echo addslashes($batch->product->name); ?>');" action="{{route('deleteBatch',$batch->id)}}" method="post">
														@csrf
														@method('DELETE')
														<button type="submit" class="btn btn-danger waves-effect waves-light">
															<i class="mdi mdi-delete"></i>
														</button>
													</form>
								</td>
							</tr>
						<?php endforeach; ?>
					</tbody>
				</table>
			</div>
			<div class="pt-3 pagination-rounded justify-content-end">
				{{ $batches->appends(request()->query())->links() }}
			</div>
		</div>

	</div>
	<!-- container -->
	
</div>

<!--Status Update Modals-->
<div class="modal fade "  tabindex="-1" role="dialog" id="batchModal">
  <div class="modal-dialog modal-lg modal-ku" role="document">
    <div class="modal-content">
      <div class="modal-header bg-danger">
        <h5 class="modal-title text-white"><i class="mdi mdi-update"></i> Update Batch</h5>
        <button type="button" class="close text-white" data-dismiss="modal" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <div class="modal-body statusBody">
      	
      </div>
    </div>
  </div>
</div>

<!--for Autocomplete-->
<!--<script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" crossorigin="anonymous"></script> -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js" crossorigin="anonymous"></script>
<script src="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js" crossorigin="anonymous"></script>
<!-- Bootstrap 4 Autocomplete -->
<script src="https://cdn.jsdelivr.net/npm/bootstrap-4-autocomplete/dist/bootstrap-4-autocomplete.min.js" crossorigin="anonymous"></script>
<!--Autocomplete Ends Here-->
<script>
var modalLoading = `<p align="center"><strong>Loading</strong></p>
					        <div class="d-flex justify-content-center">
					        	<div class="spinner-grow text-blue" role="status"></div>
						    </div>`;
						    
	$(document).on('click','.updateBatchBtn',function(){
			var url = $(this).data('url');
			
			$('#batchModal').modal('show');
			
			$.ajax({
	           type:'GET',
	           url:url,
	           data:{_token:$('meta[name="csrf-token"]').attr('content')},
	           beforeSend:function(){
	           	 $('.statusBody').html(modalLoading);
	           },
	           success:function(data) {
	           		if(data!=0){
	           			$('.statusBody').html(data);
		           		$('.statusBody').find(".datetimePicker").flatpickr({
							enableTime : !0,
							dateFormat : "d-M-Y H:i"
						});
	           		} else {
	           			$('.statusBody').html(`<div class="alert alert-warning">Something went wrong. Try again after page refresh.</div>`);
	           		}
	           		
	           }
	        });
		})
		
		$('#batchModal').on('hidden.bs.modal', function (e) {
		  $('.statusBody').html(modalLoading);
		});
		
		
		$(document).on('change','.product_id',function(){
			productId = $(this).val();
			if(productId){
				$.ajax({
					type:'POST',
					url:"{{route('getProductSkusAjax')}}/"+productId,
					data:{_token:$('[name="csrf-token"]').attr('content')},
					beforeSend:function(){
						$('.skuLoader').html(`<i class="fa fa-spinner fa-spin text-danger"></i>`);
					},
					success:function(resp){
						$('.sku_id').html(resp);
						$('.skuLoader').html('');
					}
				});
			} else {
				$('.sku_id').html(`<option value="">-- Choose --</option>`);
				$('.skuLoader').html('');
			}
			
		});
		
		function destroyValidation(){
			var bootstrapValidat = $('#batchForm').data('bootstrapValidator');
			
			if (bootstrapValidat != undefined)
			   { bootstrapValidat.destroy();}
		}
		
		//base price calculations
		$(document).on('change','.brcal',function(e){
			var parentRow = $(this).parents('tr');
			var mrp = $('.brcal.mrp');
			var mratio = $('.brcal.mratio');
			var baseRate = $('.brcal.brate');
			
			if($(this).hasClass('brate')){
				if(baseRate.val() && mratio.val()){
					mrp2 = ((parseFloat(baseRate.val())*parseFloat(mratio.val()))/100);
					if(Math.round(mrp2)!=mrp2){
						mrp2 = mrp2.toFixed(2);
					}
					
					mrp.val(parseFloat(baseRate.val())+parseFloat(mrp2));
				} else {
					mrp.val(0);
				}
				
			} else {
			
				if(mrp.val() && mratio.val()){
					brate = ((parseFloat(mrp.val())*parseFloat(mratio.val()))/100);
					if(Math.round(brate)!=brate){
						brate = brate.toFixed(2);
					}
					
					baseRate.val(parseFloat(mrp.val())+parseFloat(brate));
				} else {
					baseRate.val(0);
				}
			}
		});
		//base price calculations ends

</script>

@endsection