<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

 


Route::middleware('auth:api')->get('/user', function (Request $request) {
    return $request->user();
});

Route::post('/product/store', 'ProductApiController@storeProduct')->name('product.store');  
Route::post('/product/update', 'ProductApiController@updateProduct')->name('product.update');  
//prefixed routing for maintaining the updates
Route::prefix('v1')->group(function(){
// 	Route::post('login', 'AuthController@verifyUser');
	Route::post('login', 'AuthController@verifyUserSocial'); 
	Route::post('register', 'AuthController@register');
	Route::post('sendOtp', 'AuthController@sendOtp');
	Route::post('verifyOtp', 'AuthController@verifyOtp');
	Route::post('checkEmail', 'AuthController@checkEmail');
	
	
	
	Route::get('getSupportCentre', 'SettingsController@getSupportCentre');
	Route::get('getSettings/{type?}', 'SettingsController@getSettings');
	
	Route::get('getCountriesWithStates', 'CountriesController@index');
	Route::get('getCountries', 'CountriesController@getCountries');
	Route::get('getCountryStates/{id}', 'CountriesController@getCountryStates');
	Route::get('getStateCities/{id}', 'CountriesController@getStateCities');
	
	

	//Middleware OptionalAuth
	//For Guest Browsing
	 // * if API Request is made thru this middleware
	 // * then if the authorization is present in the header then
	 // * Auth will be available with the user's data
	Route::group(['middleware' => 'OptionalAuth'], function(){
	    
		Route::post('dashboard', 'ProductsController@appDashboard');
		Route::post('dashboard2', 'SectionsController@appDashboard');
		Route::post('productsOnSale', 'ProductsController@productsOnSale');
		//Route::post('popularProducts', 'ProductsController@popularProducts');
		Route::post('sectionProducts', 'SectionsController@sectionProducts');
		Route::post('bannerProducts', 'SectionsController@bannerProducts');
		
		Route::get('subCategoriesOfCategory/{id}', 'CategoriesController@subCategoriesOfCategory');
		Route::post('productsOfSubCategory', 'ProductsController@productsOfSubCategory');
		Route::get('productDetails/{id}', 'ProductsController@productDetails');
		
		Route::get('getFilters', 'ProductsController@getFilters');
		
		//Seller Stores
		Route::post('getSellerStoresList', 'SellersController@getSellerStoresList');
		Route::get('getSellerStoreDetail/{id}', 'SellersController@getSellerStoreDetail');
		Route::post('getAllSellerStoresList', 'SellersController@getAllSellerStoresList');
		Route::post('popularProducts', 'SellersController@popularProducts');
		Route::post('checkDeliveryPostcode', 'SellersController@checkDeliveryPostcode');
		
		
		//Culture 
		Route::get('getCulturesList', 'CultureController@getCulturesList');
		Route::get('getBrandsList', 'BrandsController@getBrandsList');
		Route::post('getBrandProducts', 'BrandsController@getBrandProducts');
		Route::post('sendNominationMail', 'DeliveryAddressesController@sendNominationMail');
		Route::post('getPostcodeList', 'DeliveryAddressesController@getPostcodeList');
		
		Route::get('getListOfAllThings', 'SellersController@getListOfAllThings');
		Route::post('searchAnything', 'SellersController@searchAnything');
		Route::post('searchByCategory', 'SellersController@searchByCategory');
		Route::post('getAllProductsList', 'ProductsController@getAllProductsList');
		Route::post('getRelatedProducts', 'ProductsController@getRelatedProducts');
		
		Route::post('/resetPassword', 'AuthController@resetPassword');
		Route::post('/sendOtpOnEmail', 'AuthController@sendOtpOnEmail');
		Route::post('sendSupportMail', 'AuthController@sendSupportMail');
		
		//API to sending email and saving data from website
		Route::post('joinOurCommunity', 'UserFormsController@joinOurCommunity');
		
	});
	
	Route::group(['middleware' => 'auth:api'], function(){
		
		//User Profile
 		Route::get('userProfile', 'AuthController@userProfile');
		Route::post('updateProfile', 'AuthController@updateProfile');
		Route::post('changePassword', 'AuthController@changePassword');
		Route::post('changeNotificationStatus', 'UsersController@changeNotificationStatus');
		
		Route::post('/logout', 'AuthController@logout');
		
		Route::post('/make-payment', 'OrdersController@pay');
		Route::get('/getSavedCards', 'OrdersController@getSavedCards');
		Route::post('/deleteCard', 'OrdersController@deleteCard');
		Route::post('/applePay', 'OrdersController@applePay');
		
		//Dashboard
		//Route::post('dashboard', 'ProductsController@appDashboard');
		
		//Products on Sale
		//Route::post('productsOnSale', 'ProductsController@productsOnSale');
		
		//popular Products
		// Route::post('popularProducts', 'ProductsController@popularProducts');
		
		//Category and Products
		// Route::get('subCategoriesOfCategory/{id}', 'CategoriesController@subCategoriesOfCategory');
		// Route::get('productsOfSubCategory/{cat_id}', 'ProductsController@productsOfSubCategory');
		// Route::get('productDetails/{id}', 'ProductsController@productDetails');
		
		//Wishlist
		Route::post('addToMyWishlist', 'ProductsController@addToMyWishlist');
		Route::get('myWishList', 'ProductsController@myWishList');
		Route::post('removeFromMyWishList', 'ProductsController@removeFromMyWishList');
		
		//cart or Bag
		Route::post('addOrUpdateCart', 'CartController@addOrUpdateCart');
		Route::get('myCart', 'CartController@myCart');
		Route::post('removeFromMyCart', 'CartController@removeFromMyCart');
		Route::post('removeWholeCart', 'CartController@removeWholeCart');
		
		//filter options
		// Route::get('getFilters', 'ProductsController@getFilters');
		
		//delivery addresses
		Route::get('getMyAddressList', 'AddressesController@getMyAddressList');
		Route::get('getMyAddressDetail/{id}', 'AddressesController@getMyAddressDetail');
		Route::post('saveMyAddress', 'AddressesController@saveMyAddress');
		Route::post('updateMyAddress', 'AddressesController@updateMyAddress');
		Route::post('deleteMyAddress/{id}', 'AddressesController@deleteMyAddress');
		Route::post('makeMyAddressDefault/{id}', 'AddressesController@makeMyAddressDefault');
		
		Route::post('getPostcodeDetail', 'AddressesController@getPostcodeDetail');
		
		//orders
		Route::post('placeMyOrder', 'OrdersController@placeMyOrder');
		Route::get('myOrderDetails/{id}', 'OrdersController@myOrderDetails');
		Route::get('myOrders', 'OrdersController@myOrders');
		Route::get('getOrderStatusDetail/{id}', 'OrdersController@getOrderStatusDetail');
		Route::post('cancelOrder', 'OrdersController@cancelOrder');
		Route::post('orderStatuses', 'OrdersController@orderStatuses');
		
		//time slots
		Route::get('getTimeSlots', 'TimeSlotController@getTimeSlots');
		Route::get('getTodayTimeSlots', 'TimeSlotController@getTodayTimeSlots');
		
		//ratings controller
		Route::post('giveRatingToSeller', 'RatingsController@giveRatingToSeller');
		Route::get('getOffersList', 'OffersController@getOffersList');
		
	});
	
});
