<?php
use App\Mail\SellerSignupMail;
use App\User;
/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/


Route::get('/', function () {
	if(Auth::check()){
		return redirect('dashboard');
	}
	$merchantLoginValue  = User::merchantLogin;
    return view('auth/merchant-login',['merchantLoginValue' => $merchantLoginValue]);
//     return view('auth/merchant-login');
})->name('/');

//Send SMS on mobile phone
Route::get('/sendSms', 'UsersController@sendSms');

//handeling when visit logout url without logged-in
Route::get('/logout', 'Auth\LoginController@logout')->name('logout' );

Route::get('/dashboard', 'DashboardController@index')->name('dashboard');
//to check if the SKU is unique or not
Route::get('validateProductSku/{id?}', 'ProductsController@validateProductSku')->name('validateProductSku');

//to check if the Batch Number is unique or not
Route::get('validateBatchNumber/{id?}', 'BatchesController@validateBatchNumber')->name('validateBatchNumber');
Route::delete('deleteBatch/{id}', 'BatchesController@deleteBatch')->name('deleteBatch');
Route::get('validateProductBatchNumber', 'ProductsController@validateProductBatchNumber')->name('validateProductBatchNumber');
Route::post('getSpecs', 'ProductsController@getSpecs')->name('getSpecs');
Route::post('getNewSku','ProductsController@getNewSku')->name('getNewSku');

Auth::routes();

//Route::resource('merchants','MerchantsController');	


Route::group(['middleware'=>['auth','VerifyPermission']],function () {
	Route::resource('products','ProductsController');
	Route::resource('categories','CategoriesController');
	Route::resource('specifications','SpecificationsController');
	Route::resource('taxes','TaxesController');
	Route::resource('orders','OrdersController');
	Route::resource('merchants','MerchantsController');
	Route::resource('users','UsersController');
	Route::resource('offers','OffersController');
	Route::resource('batches','BatchesController');
	Route::resource('stock','StockController');
	Route::resource('invoices','InvoicesController');
	Route::resource('sections','SectionsController');
	
	Route::get('sellerItems/{id}', 'StockController@sellerItems')->name('sellerItems');
	Route::get('/getEditStock/{id}','StockController@getEditStock')->name('getEditStock');
	Route::post('updateStock','StockController@updateStock')->name('updateStock');	
	Route::post('upload_stock','StockController@upload_stock')->name('upload_stock');
	Route::get('/download_csv/{file}','StockController@download_csv')->name('download_csv');	
	
	Route::post('getProductSkusAjax/{product_id?}','BatchesController@getProductSkusAjax')->name('getProductSkusAjax');
	Route::get('getEditForm/{id}','BatchesController@getEditForm')->name('getEditForm');
	Route::post('updateBatch','BatchesController@updateBatch')->name('updateBatch');	
	
	Route::post('getInvoiceDetailsAjax/{invoice_id?}','BatchesController@getInvoiceDetailsAjax')->name('getInvoiceDetailsAjax');	
	
	
	Route::get('products/duplicate/{id}', 'ProductsController@duplicate')->name('products.duplicate');
	Route::get('products/createsample', 'ProductsController@createsample')->name('products.createsample');
	
	Route::get('products/skudetails/{sku_id?}', 'ProductsController@skudetails')->name('products.skudetails');
	Route::post('updateSkuBatchOrder', 'ProductsController@updateSkuBatchOrder')->name('updateSkuBatchOrder');
		
		
	
		
	//support centre and settings
	Route::get('supportCentre', 'SettingsController@supportCentre')->name('supportCentre');
	Route::post('updateSupportCentre', 'SettingsController@updateSupportCentre')->name('updateSupportCentre');
	Route::get('settings', 'SettingsController@settings')->name('settings');
	Route::post('updateSettings', 'SettingsController@updateSettings')->name('updateSettings');
	
	//Tax Controller
	Route::get('service-charges', 'TaxesController@serviceChargesIndex')->name('service-charges');
	Route::post('update-service-charges', 'TaxesController@updateServiceCharges')->name('update-service-charges');
	
	
	Route::get('appDashboardSettings', 'CategoriesController@appDashboardSettings')->name('appDashboardSettings');
	Route::post('appDashboardSettings', 'CategoriesController@appDashboardSettings')->name('appDashboardSettings');
	// 	remove this
	Route::get('compressImage', 'CategoriesController@compressImage')->name('compressImage');
	
	Route::get('appDashboardSections', 'SectionsController@appDashboardSections')->name('appDashboardSections');
	Route::get('toggleSectionEnable/{id?}/{enabled?}', 'SectionsController@toggleSectionEnable')->name('toggleSectionEnable');
	Route::get('getCategoryProductsAjax', 'SectionsController@getCategoryProductsAjax')->name('getCategoryProductsAjax');
	Route::post('updateSectionOrderAjax', 'SectionsController@updateSectionOrderAjax')->name('updateSectionOrderAjax');
	Route::get('slidersection', 'SectionsController@slidersection')->name('sections.slider');
	Route::get('newsliderbanner', 'SectionsController@createsliderbanner')->name('sections.newbanner');
	Route::post('savebanner', 'SectionsController@savebanner')->name('savesliderbanner');
	Route::get('editsliderbanner/{id}', 'SectionsController@editsliderbanner')->name('sections.editbanner');
	Route::put('updatesliderbanner', 'SectionsController@updatesliderbanner')->name('updatesliderbanner');
	
	Route::post('updateBannerOrderAjax', 'SectionsController@updateBannerOrderAjax')->name('updateBannerOrderAjax');
	Route::post('getBannerProductsAjax', 'SectionsController@getBannerProductsAjax')->name('getBannerProductsAjax');
	Route::post('updateBannerProductOrderAjax', 'SectionsController@updateBannerProductOrderAjax')->name('updateBannerProductOrderAjax');
	Route::get('toggleBannerEnable/{id?}/{enabled?}', 'SectionsController@toggleBannerEnable')->name('toggleBannerEnable');
	Route::post('updateBannerProductOrderAjax', 'SectionsController@updateBannerProductOrderAjax')->name('updateBannerProductOrderAjax');
	Route::post('destroyBanner', 'SectionsController@destroyBanner')->name('destroyBanner');
	
	
	/*Brands Controller*/
	Route::get('brandsList', 'BrandsController@index')->name('brandsList');
	Route::get('addBrand', 'BrandsController@add')->name('addBrand');
	Route::post('saveBrand', 'BrandsController@save')->name('saveBrand');
	Route::get('editBrand/{id}', 'BrandsController@edit')->name('editBrand');
	Route::post('updateBrand/{id}', 'BrandsController@update')->name('updateBrand');
	Route::delete('deleteBrand', 'BrandsController@delete')->name('deleteBrand');
	Route::get('importBrands', 'BrandsController@importBrands')->name('importBrands');
	Route::post('saveImportBrands', 'BrandsController@saveImportBrands')->name('saveImportBrands');
	Route::get('exportSampleBrands', 'BrandsController@exportSampleBrands')->name('exportSampleBrands');
	Route::post('saveAjaxBrand', 'BrandsController@saveAjaxBrand')->name('saveAjaxBrand');
	 
	
	Route::post('updateProductBatchOrder', 'ProductsController@updateProductBatchOrder')->name('updateProductBatchOrder');
	
	Route::get('adminUsers', 'UsersController@adminUsersList')->name('adminUsersList');
	Route::get('adminUserProfle/{id}', 'UsersController@adminUserProfle')->name('adminUserProfle');
	Route::get('editAdminUser/{id}', 'UsersController@editAdminUser')->name('editAdminUser');
	Route::post('saveEditAdminUser/{id}', 'UsersController@saveEditAdminUser')->name('saveEditAdminUser');
	Route::get('addAdminUser', 'UsersController@addAdminUser')->name('addAdminUser');
	Route::post('saveNewAdminUser', 'UsersController@saveNewAdminUser')->name('saveNewAdminUser');
	Route::post('updateNotificationToken', 'UsersController@updateNotificationToken')->name('updateNotificationToken');
	Route::get('send', 'UsersController@send')->name('send');
	 
	/* Seller Action */
	Route::get('sellersList', 'SellersController@sellersList')->name('sellersList');
	Route::get('addSeller', 'SellersController@addSeller')->name('addSeller');
	Route::post('saveSeller', 'SellersController@saveSeller')->name('saveSeller');
	Route::get('sellerProfile/{id}', 'SellersController@sellerProfile')->name('sellerProfile');
	Route::get('editSeller/{id}', 'SellersController@editSeller')->name('editSeller');
	Route::post('saveEditSeller/{id}', 'SellersController@saveEditSeller')->name('saveEditSeller');
	Route::get('editSellerAddress/{id}', 'SellersController@editSellerAddress')->name('editSellerAddress');
	Route::post('saveEditSellerAddress/{id}', 'SellersController@saveEditSellerAddress')->name('saveEditSellerAddress');
	Route::post('changeStoreOpenStatus', 'SellersController@changeStoreOpenStatus')->name('changeStoreOpenStatus');
	Route::post('sellerProducts/{id}', 'SellersController@sellerProducts')->name('sellerProducts');
	Route::get('sellerProducts/{id}', 'SellersController@sellerProducts')->name('sellerProducts');
	Route::get('markStoreExist/{id}/{popular}', 'SellersController@markStoreExist')->name('markStoreExist');
	Route::get('markEnabled/{id}/{popular}', 'SellersController@markEnabled')->name('markEnabled');
	
	Route::get('newDashboard', 'SellersController@newDashboard')->name('newDashboard');
	
	// 	Route::get('sellerSkudetails/{id}/{sellerid}', 'SellersController@sellerSkudetails')->name('sellerSkudetails');
	
	Route::get('id/{id}/sellerId/{sellerId}', [
	    'as' => 'sellerSkudetails', 'uses' => 'SellersController@sellerSkudetails']);
	
	Route::get('appUsers', 'UsersController@appUsersList')->name('appUsersList');
	Route::get('appUserProfle/{id}', 'UsersController@appUserProfle')->name('appUserProfle');
	Route::get('editAppUser/{id}', 'UsersController@editAppUser')->name('editAppUser');
	Route::post('saveEditAppUser/{id}', 'UsersController@saveEditAppUser')->name('saveEditAppUser');
	Route::delete('deleteAppUser', 'UsersController@deleteAppUser')->name('deleteAppUser');
	
	
	Route::get('exportProductsSample', 'ProductsController@exportSample')->name('exportSample');
	Route::post('importProducts', 'ProductsController@importProducts')->name('importProducts');
	
	Route::get('products/importStatus', 'ProductsController@importStatus')->name('importStatus');
	Route::post('products/deleteProductImage/{id}','ProductsController@deleteProductImage')->name('deleteProductImage');
	Route::post('products/deleteProductSpecification/{id}','ProductsController@deleteProductSpecification')->name('deleteProductSpecification');
	Route::post('products/destroyProduct/{id}','ProductsController@destroyProduct')->name('destroyProduct');
	
	Route::post('deleteProductSku/{id}','ProductsController@deleteProductSku')->name('deleteProductSku');
	Route::post('deleteProductSkuAjax/{id}','ProductsController@deleteProductSkuAjax')->name('deleteProductSkuAjax');	
	Route::post('deleteImageAjax/{id}','ProductsController@deleteImageAjax')->name('deleteImageAjax');	
	Route::post('deleteSpecificationAjax/{id}','ProductsController@deleteSpecificationAjax')->name('deleteSpecificationAjax');	
	
	Route::post('/merchants/search', 'MerchantsController@index')->name('mearchant.search');
	
	//for getting with ajax
	Route::get('getMerchantProducts/{mid}','MerchantsController@getMerchantProducts')->name('getMerchantProducts');
	Route::get('getMerchantSales/{mid}','MerchantsController@getMerchantSales')->name('getMerchantSales');
	
	Route::get('rolePermissions/{role?}', 'UsersController@rolePermissions')->name('rolePermissions');
	Route::post('rolePermissions/{role?}', 'UsersController@rolePermissions')->name('rolePermissions');
	
	Route::get('markPopular/{id}/{popular}', 'ProductsController@markPopular')->name('markPopular');
	Route::get('markBatchEnabled/{id}/{enabled}', 'BatchesController@markBatchEnabled')->name('markBatchEnabled');
	Route::get('downloadBatchSample', 'BatchesController@downloadBatchSample')->name('downloadBatchSample');
	
	
	Route::post('getOrderStatusForm', 'OrdersController@getOrderStatusForm')->name('getOrderStatusForm');
	//Route::post('updateOrderProductStatus', 'OrdersController@updateOrderProductStatus')->name('updateOrderProductStatus');
	Route::post('updateOrderStatus', 'OrdersController@updateOrderStatus')->name('updateOrderStatus');
	Route::post('getOrderStatusHistory', 'OrdersController@getOrderStatusHistory')->name('getOrderStatusHistory');
	Route::get('printDeliveryNote/{id}', 'OrdersController@printDeliveryNote')->name('printDeliveryNote');
	Route::get('printInvoice/{id}', 'OrdersController@printInvoice')->name('printInvoice');
	Route::get('markOrderStatus/{id}/{type}', 'OrdersController@markOrderStatus')->name('markOrderStatus');
	
	/* Delivery Addresses */
	Route::get('deliveryAddressesList', 'DeliveryAddressesController@deliveryAddressesList')->name('deliveryAddressesList');
	Route::get('addDeliveryAddress', 'DeliveryAddressesController@addDeliveryAddress')->name('addDeliveryAddress');
	Route::post('saveDeliveryAddress', 'DeliveryAddressesController@saveDeliveryAddress')->name('saveDeliveryAddress');
	Route::get('editDeliveryAddress/{id}', 'DeliveryAddressesController@editDeliveryAddress')->name('editDeliveryAddress');
	Route::post('saveEditDeliveryAddress/{id}', 'DeliveryAddressesController@saveEditDeliveryAddress')->name('saveEditDeliveryAddress');
	Route::delete('deleteDeliveryAddress', 'DeliveryAddressesController@deleteDeliveryAddress')->name('deleteDeliveryAddress');
	Route::get('markInUse/{id}/{inUse}', 'DeliveryAddressesController@markInUse')->name('markInUse');
	Route::get('importAddresses', 'DeliveryAddressesController@importAddresses')->name('importAddresses');
	Route::post('saveNotDeliveredAddress', 'DeliveryAddressesController@saveNotDeliveredAddress')->name('saveNotDeliveredAddress');
	Route::get('exportSampleAddresses', 'DeliveryAddressesController@exportSampleAddresses')->name('exportSampleAddresses');
	
	/* Cultures Controller */
	Route::get('culturesList', 'CultureController@culturesList')->name('culturesList');
	Route::get('addCultures', 'CultureController@addCultures')->name('addCultures');
	Route::post('saveCultures', 'CultureController@saveCultures')->name('saveCultures');
	Route::get('editCulture/{id}', 'CultureController@editCulture')->name('editCulture');
	Route::post('saveEditCulture/{id}', 'CultureController@saveEditCulture')->name('saveEditCulture');
	Route::delete('deleteCulture', 'CultureController@deleteCulture')->name('deleteCulture');
	
	
	/* Time Slot */
	Route::get('timeSlotsList', 'TimeSlotController@timeSlotsList')->name('timeSlotsList');
	Route::get('addTimeSlot', 'TimeSlotController@addTimeSlot')->name('addTimeSlot');
	Route::post('saveTimeSlot', 'TimeSlotController@saveTimeSlot')->name('saveTimeSlot');
	Route::post('saveTodayTimeSlot', 'TimeSlotController@saveTodayTimeSlot')->name('saveTodayTimeSlot');
	Route::get('editTimeSlot/{id}', 'TimeSlotController@editTimeSlot')->name('editTimeSlot');
	Route::post('saveEditTimeSlot/{id}', 'TimeSlotController@saveEditTimeSlot')->name('saveEditTimeSlot');
	Route::post('saveEditTodayTimeSlot/{id}', 'TimeSlotController@saveEditTodayTimeSlot')->name('saveEditTodayTimeSlot');
	Route::delete('deleteTimeSlot', 'TimeSlotController@deleteTimeSlot')->name('deleteTimeSlot');
	Route::get('markSlotEnabled/{id}/{enb}', 'TimeSlotController@markSlotEnabled')->name('markSlotEnabled');
	
	Route::get('todayTimeSlotsList', 'TimeSlotController@todayTimeSlotsList')->name('todayTimeSlotsList');
	Route::get('addTodayTimeSlot', 'TimeSlotController@addTodayTimeSlot')->name('addTodayTimeSlot');
	
	Route::get('webSettings', 'SettingsController@webSettings')->name('webSettings');
	Route::post('updateWebSettings', 'SettingsController@updateWebSettings')->name('updateWebSettings');
	
	Route::post('getNewCouponCode', 'OffersController@getNewCouponCode')->name('getNewCouponCode');
	
	Route::get('userFormIndex', 'UserFormsController@index')->name('userFormIndex');
	Route::get('userFormView/{id}', 'UserFormsController@view')->name('userFormView');
	// My code
	Route::get('getDataByDate/', 'OrdersController@getDataByDate')->name('getDataByDate');
	Route::get('bestSellerByProduct/', 'SellersController@bestSellerByProduct')->name('bestSellerByProduct');
});


// My Routes

Route::get('/admin-login', function () {
	$adminLoginValue = User::AdminLogin;
    return view('auth/login',['adminLoginValue' => $adminLoginValue]);
});

